/*
  GLOBALS
*/
var scribble  = require('../index')
  , nock      = require('nock')
  , should    = require('should')
  , crypto    = require('crypto')
  , song      = { artist: 'Slayer', track: 'Disciple', album: 'God Hates Us All' }
//nock.recorder.rec()
/*
  FUNCTIONS
*/
/**/// Private: Make MD5 hashes
/**///
/**/// Args
/**/// input - string input to hash
/**///
/**/// Returns
/**/// return - md5 hash of the input string
function makeHash(input) {
  return crypto.createHash('md5').update(input, 'utf8').digest("hex")
}
/*
  TESTS
*/

/*
  POSTS
*/
describe('[Object creation] :', function() {
  var scrobbler = new scribble('a','a','a','a')
  // 1
  it('Should create an object with 5 properties', function() {
    scrobbler.apiKey.should.equal('a')
    scrobbler.apiSecret.should.equal('a')
    scrobbler.username.should.equal('a')
    scrobbler.password.should.equal('a')
    scrobbler.should.have.property('sessionKey')
  })
  // 2
  it('Should start with a null sessionKey', function() {
    should.not.exist(scrobbler.sessionKey)
  })
}) // [Object creation]

describe('[MakeSession] :', function() {
  var scrobbler = new scribble('a','a','a','a')
    , token   = makeHash('a' + makeHash('a'))
    , apiSig  = makeHash('api_keyaauthToken' + token + 'methodauth.getMobileSessionusernameaa')
  // 3
  it('Should start with a null sessionKey', function() {
    should.not.exist(scrobbler.sessionKey)
  })
  // 4
  it('Should end with a sessionKey', function(endTest) {
    nock('http://ws.audioscrobbler.com:80')
      .get('/2.0/?method=auth.getMobileSession&username=a&authToken=e35bce2719cee48819fe422c51bec259&api_key=a&api_sig=99cad30a83f2c090f2d3de9d80fcaabe&format=json')
      .reply(200, "{\"session\":{\"name\":\"GodOfThisAge\",\"key\":\"sweetkeybro\",\"subscriber\":\"0\"}}\n", {})
    scrobbler.MakeSession(function(key) {
      should.exist(key)
      key.should.equal('sweetkeybro')
      endTest()
    })
  })
}) // [MakeSession]

describe('[Love]', function() {
  //var scrobbler = new scribble('a','a','a','a')
}) // [Love]

describe('[Scrobble]', function() {
  // var scrobbler = new scribble('a','a','a','a')
  //   , apiSig    = makeHash('api_keyaartist' + song.artist + 'methodtrack.scrobblesksweetkeybrotimestamp1359941414track' + song.track + 'a')

  // // 412aaebbf778278ade9f4d766da11721
  // // mock key request
  // nock('http://ws.audioscrobbler.com:80')
  //     .get('/2.0/?method=auth.getMobileSession&username=a&authToken=e35bce2719cee48819fe422c51bec259&api_key=a&api_sig=99cad30a83f2c090f2d3de9d80fcaabe&format=json')
  //     .reply(200, "{\"session\":{\"name\":\"GodOfThisAge\",\"key\":\"sweetkeybro\",\"subscriber\":\"0\"}}\n", {})
  // // mock post request
  // nock('http://ws.audioscrobbler.com:80')
  // //.post('/2.0/', "method=track.scrobble&api_key=a&sk=sweetkeybro&api_sig=412aaebbf778278ade9f4d766da11721&timestamp=1359941414&artist=Slayer&track=Disciple")
  // .reply(200, "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<lfm status=\"ok\">\n<scrobble>\n    <track corrected=\"0\">Disciple</track>\n    <artist corrected=\"0\">Slayer</artist>\n    <album corrected=\"0\"></album>\n    <albumArtist corrected=\"0\"></albumArtist>\n    <ignoredMessage code=\"0\"></ignoredMessage>\n</nowplaying></lfm>\n", { date: 'Mon, 04 Feb 2013 01:04:34 GMT',
  // server: 'Apache/2.2.22 (Unix)',
  // 'x-web-node': 'www216',
  // 'access-control-allow-origin': '*',
  // 'access-control-allow-methods': 'POST, GET, OPTIONS',
  // 'access-control-max-age': '86400',
  // 'content-length': '301',
  // connection: 'close',
  // 'content-type': 'text/xml; charset=utf-8;' })

  // TODO
  // Figure out a way around the timestamp required
  // it('Should auto generate a key and return the post response', function(endTest) {
  //   scrobbler.Scrobble(song, function(ret) {
  //     ret.should.not.equal('')
  //     ret.should.equal("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<lfm status=\"ok\">\n<scrobble>\n    <track corrected=\"0\">Disciple</track>\n    <artist corrected=\"0\">Slayer</artist>\n    <album corrected=\"0\"></album>\n    <albumArtist corrected=\"0\"></albumArtist>\n    <ignoredMessage code=\"0\"></ignoredMessage>\n</nowplaying></lfm>\n")
  //     endTest()
  //   })
  // })
}) // [Scrobble]

describe('[Now Playing]', function() {
  var scrobbler = new scribble('a','a','a','a')
    , apiSig  = makeHash('api_keyaartistSlayermethodtrack.updateNowPlayingsksweetkeybrotrackDisciplea')
  // mock key request
  nock('http://ws.audioscrobbler.com:80')
      .get('/2.0/?method=auth.getMobileSession&username=a&authToken=e35bce2719cee48819fe422c51bec259&api_key=a&api_sig=99cad30a83f2c090f2d3de9d80fcaabe&format=json')
      .reply(200, "{\"session\":{\"name\":\"GodOfThisAge\",\"key\":\"sweetkeybro\",\"subscriber\":\"0\"}}\n", {})
  // mock post request
  nock('http://ws.audioscrobbler.com:80')
  .post('/2.0/', "method=track.updateNowPlaying&api_key=a&sk=sweetkeybro&api_sig=fae2ed6a254ac0f07cc838f3d676d0ab&artist=Slayer&track=Disciple")
  .reply(200, "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<lfm status=\"ok\">\n<nowplaying>\n    <track corrected=\"0\">Disciple</track>\n    <artist corrected=\"0\">Slayer</artist>\n    <album corrected=\"0\"></album>\n    <albumArtist corrected=\"0\"></albumArtist>\n    <ignoredMessage code=\"0\"></ignoredMessage>\n</nowplaying></lfm>\n", { date: 'Mon, 04 Feb 2013 01:04:34 GMT',
  server: 'Apache/2.2.22 (Unix)',
  'x-web-node': 'www216',
  'access-control-allow-origin': '*',
  'access-control-allow-methods': 'POST, GET, OPTIONS',
  'access-control-max-age': '86400',
  'content-length': '301',
  connection: 'close',
  'content-type': 'text/xml; charset=utf-8;' })

  // 5
  it('Should auto generate a key and return the post response', function(endTest) {
    scrobbler.NowPlaying(song, function(ret) {
      ret.should.not.equal('')
      ret.should.equal("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n<lfm status=\"ok\">\n<nowplaying>\n    <track corrected=\"0\">Disciple</track>\n    <artist corrected=\"0\">Slayer</artist>\n    <album corrected=\"0\"></album>\n    <albumArtist corrected=\"0\"></albumArtist>\n    <ignoredMessage code=\"0\"></ignoredMessage>\n</nowplaying></lfm>\n")
      endTest()
    })
  })
}) // [Now Playing]

/*
  GETS
*/
/*<><><><><><><><><><><><><><><><><><><>
              ARTIST
<><><><><><><><><><><><><><><><><><><>*/
describe('[Artist Info]', function() {
  // 6
  it('should return the artist info', function(endTest) {
    // mock request
    nock('http://ws.audioscrobbler.com:80')
        .get('/2.0/?method=artist.getInfo&artist=' + song.artist + '&api_key=a&format=json')
        .reply(200, {"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/8428755.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/8428755.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/8428755.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/8428755.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/8428755\/Slayer.jpg","size":"mega"}],"streamable":"1","ontour":"1","stats":{"listeners":"1143280","playcount":"58999948"},"similar":{"artist":[{"name":"Exodus","url":"http:\/\/www.last.fm\/music\/Exodus","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/11095665.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/11095665.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/11095665.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/11095665.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/11095665\/Exodus.jpg","size":"mega"}]},{"name":"Kreator","url":"http:\/\/www.last.fm\/music\/Kreator","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/49323497.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/49323497.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/49323497.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/49323497.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/49323497\/Kreator.jpg","size":"mega"}]},{"name":"Sodom","url":"http:\/\/www.last.fm\/music\/Sodom","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/29588545.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/29588545.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/29588545.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/29588545.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/29588545\/Sodom+92_02.jpg","size":"mega"}]},{"name":"Megadeth","url":"http:\/\/www.last.fm\/music\/Megadeth","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/31195359.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/31195359.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/31195359.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/31195359.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/31195359\/Megadeth+censored.jpg","size":"mega"}]},{"name":"Anthrax","url":"http:\/\/www.last.fm\/music\/Anthrax","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/82890667.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/82890667.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/82890667.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/82890667.png","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/82890667\/Anthrax+++Band+PhotoPNG+Austral.png","size":"mega"}]}]},"tags":{"tag":[{"name":"thrash metal","url":"http:\/\/www.last.fm\/tag\/thrash%20metal"},{"name":"metal","url":"http:\/\/www.last.fm\/tag\/metal"},{"name":"speed metal","url":"http:\/\/www.last.fm\/tag\/speed%20metal"},{"name":"heavy metal","url":"http:\/\/www.last.fm\/tag\/heavy%20metal"},{"name":"thrash","url":"http:\/\/www.last.fm\/tag\/thrash"}]},"bio":{"published":"Mon, 24 May 2010 15:32:09 +0000","summary":"Slayer is a <a href=\"http:\/\/www.last.fm\/tag\/thrash%20metal\" class=\"bbcode_tag\" rel=\"tag\">thrash metal<\/a> band from US, formed in 1981. The band was founded by guitarists <a href=\"http:\/\/www.last.fm\/music\/Jeff+Hanneman\" class=\"bbcode_artist\">Jeff Hanneman<\/a> and <a href=\"http:\/\/www.last.fm\/music\/Kerry+King\" class=\"bbcode_artist\">Kerry King<\/a>. The band was credited as one of the &quot;Big Four&quot; <a href=\"http:\/\/www.last.fm\/tag\/thrash%20metal\" class=\"bbcode_tag\" rel=\"tag\">thrash metal<\/a> bands, along with <a href=\"http:\/\/www.last.fm\/music\/Metallica\" class=\"bbcode_artist\">Metallica<\/a>, <a href=\"http:\/\/www.last.fm\/music\/Anthrax\" class=\"bbcode_artist\">Anthrax<\/a>, and <a href=\"http:\/\/www.last.fm\/music\/Megadeth\" class=\"bbcode_artist\">Megadeth<\/a>.  Slayer's musical traits involve fast tremolo picking, guitar solos, double bass drumming, and shouting vocals. The band's lyrics and album art, which cover topics such as death, deviants, suicide, genocide, necrophilia, insanity, Nazism, religion, Satanism...","content":"<strong>Slayer<\/strong> is a <a href=\"http:\/\/www.last.fm\/tag\/thrash%20metal\" class=\"bbcode_tag\" rel=\"tag\">thrash metal<\/a> band from US, formed in 1981. The band was founded by guitarists <a href=\"http:\/\/www.last.fm\/music\/Jeff+Hanneman\" class=\"bbcode_artist\">Jeff Hanneman<\/a> and <a href=\"http:\/\/www.last.fm\/music\/Kerry+King\" class=\"bbcode_artist\">Kerry King<\/a>. The band was credited as one of the &quot;Big Four&quot; <a href=\"http:\/\/www.last.fm\/tag\/thrash%20metal\" class=\"bbcode_tag\" rel=\"tag\">thrash metal<\/a> bands, along with <a href=\"http:\/\/www.last.fm\/music\/Metallica\" class=\"bbcode_artist\">Metallica<\/a>, <a href=\"http:\/\/www.last.fm\/music\/Anthrax\" class=\"bbcode_artist\">Anthrax<\/a>, and <a href=\"http:\/\/www.last.fm\/music\/Megadeth\" class=\"bbcode_artist\">Megadeth<\/a>.\n \n Slayer's musical traits involve fast tremolo picking, guitar solos, double bass drumming, and shouting vocals. The band's lyrics and album art, which cover topics such as death, deviants, suicide, genocide, necrophilia, insanity, Nazism, religion, Satanism, serial killers, and warfare have generated album bans, delays, lawsuits and strong criticism from religious groups and the general public. Slayer is best known for speaking through perspective without being necessarily sympathetic to the cause of their inspiration. \n \n Since their debut record in 1983, the band has released two live albums, one cover album, one box set, three DVDs, one VHS, two EPs, and eleven studio albums, four of which have received gold certification in the United States. The band has received three Grammy nominations, winning one in 2007 for the song &quot;<a title=\"Slayer &ndash; Eyes Of The Insane\" href=\"http:\/\/www.last.fm\/music\/Slayer\/_\/Eyes+Of+The+Insane\" class=\"bbcode_track\">Eyes Of The Insane<\/a>&quot;, and one in 2008 for the song &quot;<a title=\"Slayer &ndash; Final Six\" href=\"http:\/\/www.last.fm\/music\/Slayer\/_\/Final+Six\" class=\"bbcode_track\">Final Six<\/a>&quot;. They have headlined music festivals worldwide, including Unholy Alliance and Ozzfest.\n \n <strong>Slayer<\/strong> was formed in 1981 in Huntington Park, Los Angeles, California. The Band was formed by guitarists <a href=\"http:\/\/www.last.fm\/music\/Jeff+Hanneman\" class=\"bbcode_artist\">Jeff Hanneman<\/a> and <a href=\"http:\/\/www.last.fm\/music\/Kerry+King\" class=\"bbcode_artist\">Kerry King<\/a> when they met at a tryout for a local band. <a href=\"http:\/\/www.last.fm\/music\/Tom+Araya\" class=\"bbcode_artist\">Tom Araya<\/a> soon joined on bass and vocals, and drummer <a href=\"http:\/\/www.last.fm\/music\/Dave+Lombardo\" class=\"bbcode_artist\">Dave Lombardo<\/a> joined last later in 1982. They played in a style reminiscent of early <a href=\"http:\/\/www.last.fm\/music\/Exodus\" class=\"bbcode_artist\">Exodus<\/a>, influenced heavily by punk acts <a href=\"http:\/\/www.last.fm\/music\/Minor+Threat\" class=\"bbcode_artist\">Minor Threat<\/a>, <a href=\"http:\/\/www.last.fm\/music\/+noredirect\/Stooges\" class=\"bbcode_artist\">Stooges<\/a>  and Heavy Metal bands <a href=\"http:\/\/www.last.fm\/music\/Venom\" class=\"bbcode_artist\">Venom<\/a> and <a href=\"http:\/\/www.last.fm\/music\/Judas+Priest\" class=\"bbcode_artist\">Judas Priest<\/a>.\n \n &quot;<em><a title=\"Slayer &ndash; Black Magic\" href=\"http:\/\/www.last.fm\/music\/Slayer\/_\/Black+Magic\" class=\"bbcode_track\">Black Magic<\/a><\/em>&quot;, the heaviest, darkest, and most evil song at the time (according to some), was written in February, 1983, and marked <strong>Slayer<\/strong>'s definite venture into <a href=\"http:\/\/www.last.fm\/tag\/thrash%20metal\" class=\"bbcode_tag\" rel=\"tag\">thrash metal<\/a>. They secured a spot on the compilation appeareance &quot;<em><a href=\"http:\/\/www.last.fm\/music\/Various+Artists\/Metal+Massacre%2C+Volume+3\">Metal Massacre III<\/a><\/em>&quot; in 1983, contributing the <a href=\"http:\/\/www.last.fm\/tag\/speed%20metal\" class=\"bbcode_tag\" rel=\"tag\">speed metal<\/a> song &quot;Aggressive Perfector&quot;. They also recorded two demos in 1983, and became a popular live act in the Los Angeles area. (In 1983, Bob Gourley filled in for Dave for one gig. He later appeared in <a href=\"http:\/\/www.last.fm\/music\/Dark+Angel\" class=\"bbcode_artist\">Dark Angel<\/a> (US).) Their debut LP, <em><a title=\"Slayer - Show No Mercy\" href=\"http:\/\/www.last.fm\/music\/Slayer\/Show+No+Mercy\" class=\"bbcode_album\">Show No Mercy<\/a><\/em>, was recorded in November, 1983.)\n \n A live EP was recorded in the studio in front of 50 of their closest friends in January, 1984, and after that &quot;Show No Mercy&quot; was released. The &quot;<em><a title=\"Slayer - Haunting the Chapel\" href=\"http:\/\/www.last.fm\/music\/Slayer\/Haunting+the+Chapel\" class=\"bbcode_album\">Haunting the Chapel<\/a><\/em>&quot; and &quot;<em><a title=\"Slayer - Live Undead\" href=\"http:\/\/www.last.fm\/music\/Slayer\/Live+Undead\" class=\"bbcode_album\">Live Undead<\/a><\/em>&quot; EPs followed.\n \n After 1985's &quot;<em><a title=\"Slayer - Hell Awaits\" href=\"http:\/\/www.last.fm\/music\/Slayer\/Hell+Awaits\" class=\"bbcode_album\">Hell Awaits<\/a><\/em>&quot;, <strong>Slayer<\/strong> were signed to Rick Rubin's Def Jam record label, better known for acts like <a href=\"http:\/\/www.last.fm\/music\/+noredirect\/Run+DMC\" class=\"bbcode_artist\">Run DMC<\/a>. This led to three studio albums, including the brutal &quot;<em><a title=\"Slayer - Reign in Blood\" href=\"http:\/\/www.last.fm\/music\/Slayer\/Reign+in+Blood\" class=\"bbcode_album\">Reign in Blood<\/a><\/em>&quot;, as well as a 1991 live double CD, &quot;<em><a title=\"Slayer - Decade Of Aggression\" href=\"http:\/\/www.last.fm\/music\/Slayer\/Decade+Of+Aggression\" class=\"bbcode_album\">Decade Of Aggression<\/a><\/em>&quot;.\n \n <a href=\"http:\/\/www.last.fm\/music\/Dave+Lombardo\" class=\"bbcode_artist\">Dave Lombardo<\/a> left the band in 1987 for several weeks, and was replaced for a few live gigs by Tony Scaglione (<a href=\"http:\/\/www.last.fm\/music\/Whiplash\" class=\"bbcode_artist\">Whiplash<\/a> (US)), and then again left in 1992. Finally, in 1994, Paul Bostaph from <a href=\"http:\/\/www.last.fm\/music\/Forbidden\" class=\"bbcode_artist\">Forbidden<\/a> was brought in, and three more albums were recorded.\n \n Then, in 2001, Bostaph left to join <a href=\"http:\/\/www.last.fm\/music\/Systematic\" class=\"bbcode_artist\">Systematic<\/a>, and Lombardo, who had been in <a href=\"http:\/\/www.last.fm\/music\/Grip+Inc.\" class=\"bbcode_artist\">Grip Inc.<\/a> and <a href=\"http:\/\/www.last.fm\/music\/+noredirect\/Fantomas\" class=\"bbcode_artist\">Fantomas<\/a> rejoined the band.\n \n <strong>Interesting facts:<\/strong>\n \n It's official that <strong>Slayer<\/strong> is the most covered metal band in the world aside from <a href=\"http:\/\/www.last.fm\/music\/Metallica\" class=\"bbcode_artist\">Metallica<\/a> and <a href=\"http:\/\/www.last.fm\/music\/Iron+Maiden\" class=\"bbcode_artist\">Iron Maiden<\/a>.\n \n Tom Araya (born Tomas Enrique Araya June 6, 1961 in Vina del Mar, Chile. He has provided guest vocals to recordings by other artists, including &quot;Iron Gland&quot; from <a href=\"http:\/\/www.last.fm\/music\/Alice+in+Chains\" class=\"bbcode_artist\">Alice in Chains<\/a>' album Dirt, and &quot;Terrorist&quot; from <a href=\"http:\/\/www.last.fm\/music\/Soulfly\" class=\"bbcode_artist\">Soulfly<\/a>'s album Primitive. His brother is John Araya (<a href=\"http:\/\/www.last.fm\/music\/Bloodcum\" class=\"bbcode_artist\">Bloodcum<\/a> (US), <a href=\"http:\/\/www.last.fm\/music\/Thine+Eyes+Bleed\" class=\"bbcode_artist\">Thine Eyes Bleed<\/a>)\n \n Kerry King recorded a guitar solo for &quot;No Sleep Til' Brooklyn&quot; by the popular <a href=\"http:\/\/www.last.fm\/tag\/hip-hop\" class=\"bbcode_tag\" rel=\"tag\">hip-hop<\/a>\/<a href=\"http:\/\/www.last.fm\/tag\/rock\" class=\"bbcode_tag\" rel=\"tag\">rock<\/a> group <a href=\"http:\/\/www.last.fm\/music\/Beastie+Boys\" class=\"bbcode_artist\">Beastie Boys<\/a>.\n \n <a href=\"http:\/\/www.last.fm\/music\/Tom+Araya\" class=\"bbcode_artist\">Tom Araya<\/a>'s brother John (<a href=\"http:\/\/www.last.fm\/music\/Bloodcum\" class=\"bbcode_artist\">Bloodcum<\/a> (US), <a href=\"http:\/\/www.last.fm\/music\/Thine+Eyes+Bleed\" class=\"bbcode_artist\">Thine Eyes Bleed<\/a>) worked as an sound engineer for <strong>Slayer<\/strong> in their early days.\n \n According the book of Jarek Szubrycht (Lux Occulta) &quot;Bez litosci. Prawdziwa historia zespo?u <strong>Slayer<\/strong>&quot;, Greg Hall was asked to play with <strong>Slayer<\/strong> after Tony Scaglione left the band in January 1987. It never happens because Lombardo returned to the band.\n \n Gene Hoglan, the legendary drummer who has played in bands such as <a href=\"http:\/\/www.last.fm\/music\/Dark+Angel\" class=\"bbcode_artist\">Dark Angel<\/a> (US), <a href=\"http:\/\/www.last.fm\/music\/Strapping+Young+Lad\" class=\"bbcode_artist\">Strapping Young Lad<\/a> and <a href=\"http:\/\/www.last.fm\/music\/Death\" class=\"bbcode_artist\">Death<\/a>, used to be a <strong>Slayer<\/strong> lights man and roadie on the Show No Mercy tour, and was asked to be <a href=\"http:\/\/www.last.fm\/music\/Dave+Lombardo\" class=\"bbcode_artist\">Dave Lombardo<\/a>'s drum tech on the Haunting North America tour ('84), but knocked it back to pursue his own music career. He even did backup vocals on the album Show No Mercy (the song Evil Has No Boundaries).\n \n Birth Dates:\n Tom Araya (06 June 1961 Vina del Mar, Chile)\n Jeff Hannemann (31 January 1964 Oakland, CA)\n Kerry King (03 June 1964 Los Angeles, CA)\n Dave Lombardo (16 February 1965 Havana, Cuba)\n Paul Bostaph (26 March 1964 Newark, CA)+\n \n Rumor has it the band was originally known as Dragonslayer, after the 1981 movie of the same name. However, when King was asked &quot;How did you come up with the name Dragonslayer?&quot; King responded &quot;We never did; it's a myth to this day.&quot;\n    \nUser-contributed text is available under the Creative Commons By-SA License and may also be available under the GNU FDL.","placeformed":"US","yearformed":"1981","formationlist":{"formation":{"yearfrom":"1981","yearto":""}}}}},{})
    var scrobbler = new scribble('a','a','a','a')
    scrobbler.GetArtistInfo(song.artist, function(ret) {
      ret.should.be.a('object').and.have.property('artist')
      endTest()
    })
  })
}) // [Artist Info]

describe('[Similar Artists]', function() {
  // 7
  it('should return similar artists', function(endTest) {
    // mock request
    nock('http://ws.audioscrobbler.com:80')
        .get('/2.0/?method=artist.getSimilar&artist=' + song.artist + '&api_key=a&format=json&limit=50')
        .reply(200, {"similarartists":{"artist":[{"name":"Exodus","mbid":"1067b675-7637-48e4-9085-0378526b357a","match":"1","url":"www.last.fm\/music\/Exodus","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/11095665.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/11095665.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/11095665.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/11095665.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/11095665\/Exodus.jpg","size":"mega"}],"streamable":"1"},{"name":"Kreator","mbid":"39822e8d-f24e-4f07-b51b-28b22e59fbdb","match":"0.958416","url":"www.last.fm\/music\/Kreator","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/49323497.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/49323497.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/49323497.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/49323497.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/49323497\/Kreator.jpg","size":"mega"}],"streamable":"1"},{"name":"Sodom","mbid":"2251718f-7b5c-4a92-9267-895125c594d3","match":"0.938595","url":"www.last.fm\/music\/Sodom","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/29588545.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/29588545.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/29588545.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/29588545.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/29588545\/Sodom+92_02.jpg","size":"mega"}],"streamable":"1"},{"name":"Testament","mbid":"05106775-5d45-4131-aecc-1177f813ba11","match":"0.877265","url":"www.last.fm\/music\/Testament","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/24780115.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/24780115.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/24780115.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/24780115.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/24780115\/Testament+test2.jpg","size":"mega"}],"streamable":"1"},{"name":"Megadeth","mbid":"a9044915-8be3-4c7e-b11f-9e2d2ea0a91e","match":"0.859739","url":"www.last.fm\/music\/Megadeth","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/31195359.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/31195359.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/31195359.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/31195359.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/31195359\/Megadeth+censored.jpg","size":"mega"}],"streamable":"1"},{"name":"Anthrax","mbid":"315126eb-86bc-475c-a308-b8edfdd6c08c","match":"0.843889","url":"www.last.fm\/music\/Anthrax","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/82890667.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/82890667.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/82890667.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/82890667.png","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/82890667\/Anthrax+++Band+PhotoPNG+Austral.png","size":"mega"}],"streamable":"1"},{"name":"Pantera","mbid":"541f16f5-ad7a-428e-af89-9fa1b16d3c9c","match":"0.84011","url":"www.last.fm\/music\/Pantera","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/11049695.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/11049695.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/11049695.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/11049695.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/11049695\/Pantera+10978584550px.jpg","size":"mega"}],"streamable":"1"},{"name":"Sepultura","mbid":"1d93c839-22e7-4f76-ad84-d27039efc048","match":"0.78318","url":"www.last.fm\/music\/Sepultura","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/311426.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/311426.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/311426.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/311426.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/311426\/Sepultura.jpg","size":"mega"}],"streamable":"1"},{"name":"Destruction","mbid":"371f152d-1643-4b54-b32b-dd13d4c23442","match":"0.746053","url":"www.last.fm\/music\/Destruction","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/49713511.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/49713511.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/49713511.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/49713511.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/49713511\/Destruction+_6.jpg","size":"mega"}],"streamable":"1"},{"name":"Cannibal Corpse","mbid":"af8e4cc5-ef54-458d-a194-7b210acf638f","match":"0.711913","url":"www.last.fm\/music\/Cannibal+Corpse","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/28436897.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/28436897.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/28436897.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/28436897.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/28436897\/Cannibal+Corpse.jpg","size":"mega"}],"streamable":"1"},{"name":"Vader","mbid":"702c7d71-7559-4850-b6dc-1de54ada4abc","match":"0.676835","url":"www.last.fm\/music\/Vader","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/33282135.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/33282135.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/33282135.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/33282135.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/33282135\/Vader.jpg","size":"mega"}],"streamable":"1"},{"name":"Death","mbid":"5bddb294-e04c-4d8f-b9bb-e3b551802ae4","match":"0.626225","url":"www.last.fm\/music\/Death","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/51060443.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/51060443.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/51060443.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/51060443.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/51060443\/Death+chuck.jpg","size":"mega"}],"streamable":"1"},{"name":"Venom","mbid":"5ddddef1-fd5a-4ca8-8e89-df4adff4239b","match":"0.618718","url":"www.last.fm\/music\/Venom","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/39554309.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/39554309.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/39554309.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/39554309.png","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/39554309\/Venom++1981.png","size":"mega"}],"streamable":"1"},{"name":"Overkill","mbid":"97fa7dd1-4706-4fad-b6ff-e099a37837fc","match":"0.577629","url":"www.last.fm\/music\/Overkill","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/32165409.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/32165409.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/32165409.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/32165409.png","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/32165409\/Overkill.png","size":"mega"}],"streamable":"1"},{"name":"Metallica","mbid":"65f4f0c5-ef9e-490c-aee3-909e7ae6b2ab","match":"0.560108","url":"www.last.fm\/music\/Metallica","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/7560695.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/7560695.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/7560695.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/7560695.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/7560695\/Metallica++04.jpg","size":"mega"}],"streamable":"1"},{"name":"Nuclear Assault","mbid":"ffd3ac37-4d0a-42a1-a41e-e160cf60dca5","match":"0.552258","url":"www.last.fm\/music\/Nuclear+Assault","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/30947741.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/30947741.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/30947741.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/30947741.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/30947741\/Nuclear+Assault.jpg","size":"mega"}],"streamable":"1"},{"name":"Behemoth","mbid":"69632f9c-31eb-43d5-a864-e73ac72efc66","match":"0.52186","url":"www.last.fm\/music\/Behemoth","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/54196161.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/54196161.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/54196161.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/54196161.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/54196161\/Behemoth.jpg","size":"mega"}],"streamable":"1"},{"name":"Tankard","mbid":"619ce797-ddb3-4411-9e42-30eafe2ca5ec","match":"0.511709","url":"www.last.fm\/music\/Tankard","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/26648797.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/26648797.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/26648797.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/26648797.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/26648797\/Tankard+l_32d66a4829f0485d796f2847c0a0.jpg","size":"mega"}],"streamable":"1"},{"name":"Morbid Angel","mbid":"4355a36d-805e-478d-8589-64f0292724b2","match":"0.510604","url":"www.last.fm\/music\/Morbid+Angel","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/5700785.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/5700785.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/5700785.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/5700785.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/5700785\/Morbid+Angel.jpg","size":"mega"}],"streamable":"1"},{"name":"Mot\u00f6rhead","mbid":"57961a97-3796-4bf7-9f02-a985a8979ae9","match":"0.502354","url":"www.last.fm\/music\/Mot%C3%B6rhead","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/36078365.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/36078365.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/36078365.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/36078365.png","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/36078365\/Motrhead+motorhead.png","size":"mega"}],"streamable":"1"},{"name":"Iron Maiden","mbid":"7c3762a3-51f8-4cf3-8565-1ee26a90efe2","match":"0.474255","url":"www.last.fm\/music\/Iron+Maiden","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/330947.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/330947.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/330947.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/330947.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/330947\/Iron+Maiden.jpg","size":"mega"}],"streamable":"1"},{"name":"Black Sabbath","mbid":"5182c1d9-c7d2-4dad-afa0-ccfeada921a8","match":"0.46467","url":"www.last.fm\/music\/Black+Sabbath","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/27904353.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/27904353.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/27904353.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/27904353.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/27904353\/Black+Sabbath+sabbath+1970.jpg","size":"mega"}],"streamable":"1"},{"name":"Acid Drinkers","mbid":"3e103047-217b-496b-b54e-1e2db4dd58b3","match":"0.44035","url":"www.last.fm\/music\/Acid+Drinkers","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/153809.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/153809.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/153809.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/153809.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/153809\/Acid+Drinkers.jpg","size":"mega"}],"streamable":"0"},{"name":"Death Angel","mbid":"562ff334-c34f-45b0-a519-e5d171482df2","match":"0.440167","url":"www.last.fm\/music\/Death+Angel","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/51733969.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/51733969.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/51733969.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/51733969.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/51733969\/Death+Angel+Main+Photo.jpg","size":"mega"}],"streamable":"1"},{"name":"Judas Priest","mbid":"6b335658-22c8-485d-93de-0bc29a1d0349","match":"0.437822","url":"www.last.fm\/music\/Judas+Priest","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/45476153.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/45476153.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/45476153.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/45476153.png","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/45476153\/Judas+Priest.png","size":"mega"}],"streamable":"1"},{"name":"Annihilator","mbid":"5dadb7e6-864f-4107-b6d6-dfd1b6d50830","match":"0.43695","url":"www.last.fm\/music\/Annihilator","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/12024631.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/12024631.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/12024631.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/12024631.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/12024631\/Annihilator+annih7.jpg","size":"mega"}],"streamable":"1"},{"name":"Deicide","mbid":"0bd96fa4-9835-49b0-a1f2-7421e6a79dfa","match":"0.420508","url":"www.last.fm\/music\/Deicide","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/43471883.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/43471883.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/43471883.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/43471883.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/43471883\/Deicide+old.jpg","size":"mega"}],"streamable":"1"},{"name":"Machine Head","mbid":"22a1f4d7-81ff-4216-ac06-3387da6bc118","match":"0.416546","url":"www.last.fm\/music\/Machine+Head","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/68561880.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/68561880.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/68561880.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/68561880.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/68561880\/Machine+Head+FuckinEpic.jpg","size":"mega"}],"streamable":"1"},{"name":"Obituary","mbid":"2e8ac3ed-2a89-4377-ba80-790106aba554","match":"0.414504","url":"www.last.fm\/music\/Obituary","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/350885.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/350885.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/350885.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/350885.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/350885\/Obituary.jpg","size":"mega"}],"streamable":"0"},{"name":"Lamb of God","mbid":"298909e4-ebcb-47b8-95e9-cc53b087fc65","match":"0.408471","url":"www.last.fm\/music\/Lamb+of+God","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/49319591.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/49319591.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/49319591.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/49319591.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/49319591\/Lamb+of+God.jpg","size":"mega"}],"streamable":"1"},{"name":"Soulfly","mbid":"832a43c7-aa7d-439b-a6b4-4f1afa671c24","match":"0.390153","url":"www.last.fm\/music\/Soulfly","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/83134373.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/83134373.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83134373.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/83134373.png","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/83134373\/Soulfly+_2012.png","size":"mega"}],"streamable":"1"},{"name":"Municipal Waste","mbid":"b5a31e9b-1fa2-45f9-91f5-b3a25fb38038","match":"0.386221","url":"www.last.fm\/music\/Municipal+Waste","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/39169527.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/39169527.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/39169527.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/39169527.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/39169527\/Municipal+Waste+Live+at+NY.jpg","size":"mega"}],"streamable":"1"},{"name":"Amon Amarth","mbid":"5b687684-ad34-4a9f-b425-0e7aa81fbd38","match":"0.378738","url":"www.last.fm\/music\/Amon+Amarth","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/28561319.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/28561319.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/28561319.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/28561319.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/28561319\/Amon+Amarth+AAvikings.jpg","size":"mega"}],"streamable":"1"},{"name":"Arch Enemy","mbid":"e631bb92-3e2b-43e3-a2cb-b605e2fb53bd","match":"0.371403","url":"www.last.fm\/music\/Arch+Enemy","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/66167874.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/66167874.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/66167874.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/66167874.png","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/66167874\/Arch+Enemy+++2009.png","size":"mega"}],"streamable":"1"},{"name":"Black Label Society","mbid":"a8e935c6-3fcc-414c-900c-77e8170e7e7c","match":"0.367219","url":"www.last.fm\/music\/Black+Label+Society","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/48173665.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/48173665.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/48173665.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/48173665.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/48173665\/Black+Label+Society+BLS.jpg","size":"mega"}],"streamable":"1"},{"name":"S.O.D.","mbid":"b3499935-1107-4b33-8735-7b7a4511ff0d","match":"0.366493","url":"www.last.fm\/music\/S.O.D.","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/30292.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/30292.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/30292.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/30292.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/30292\/SOD.jpg","size":"mega"}],"streamable":"0"},{"name":"Kat","mbid":"5bba9d6a-5f91-4a00-9bfb-9c8257ddedbb","match":"0.366189","url":"www.last.fm\/music\/Kat","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/37750883.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/37750883.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/37750883.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/37750883.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/37750883\/Kat+zespol4.jpg","size":"mega"}],"streamable":"1"},{"name":"Violator","mbid":"2cf440f7-0527-4ad3-8a30-50bac845a6ce","match":"0.366102","url":"www.last.fm\/music\/Violator","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/56433605.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/56433605.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/56433605.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/56433605.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/56433605\/Violator+Thrashin+Joao+Pessoa+Vitria.jpg","size":"mega"}],"streamable":"1"},{"name":"Carcass","mbid":"d50a4b89-ff1f-4659-9fde-f76f8d5b3c89","match":"0.358267","url":"www.last.fm\/music\/Carcass","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/2439082.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/2439082.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/2439082.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/2439082.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/2439082\/Carcass+AllCDCovers__wake_up_an.jpg","size":"mega"}],"streamable":"1"},{"name":"Possessed","mbid":"1a7891f5-8c69-4f99-ba77-104d9800d036","match":"0.340297","url":"www.last.fm\/music\/Possessed","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/8757.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/8757.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/8757.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/8757.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/8757\/Possessed.jpg","size":"mega"}],"streamable":"1"},{"name":"Bathory","mbid":"d292b4cd-37bd-4a88-a823-8710f273cc31","match":"0.339618","url":"www.last.fm\/music\/Bathory","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/408319.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/408319.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/408319.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/408319.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/408319\/Bathory.jpg","size":"mega"}],"streamable":"1"},{"name":"Cavalera Conspiracy","mbid":"5bd1060b-8608-4817-93ca-207f7c828e2f","match":"0.32947","url":"www.last.fm\/music\/Cavalera+Conspiracy","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/3396460.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/3396460.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/3396460.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/3396460.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/3396460\/Cavalera+Conspiracy+CAVL_004_Y.jpg","size":"mega"}],"streamable":"0"},{"name":"Dark Angel","mbid":"1dd1f238-839c-4609-9bcb-5f866c82b3e8","match":"0.318094","url":"www.last.fm\/music\/Dark+Angel","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/55966619.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/55966619.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/55966619.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/55966619.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/55966619\/Dark+Angel++in+1991.jpg","size":"mega"}],"streamable":"1"},{"name":"D.R.I.","mbid":"95f6e954-8bed-4f8e-a53e-d155bb6f7ace","match":"0.312563","url":"www.last.fm\/music\/D.R.I.","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/35338569.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/35338569.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/35338569.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/35338569.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/35338569\/DRI+86PromoShot.jpg","size":"mega"}],"streamable":"1"},{"name":"Six Feet Under","mbid":"5a8688d2-696f-4060-9dd0-f772886ff95b","match":"0.311606","url":"www.last.fm\/music\/Six+Feet+Under","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/55125211.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/55125211.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/55125211.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/55125211.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/55125211\/Six+Feet+Under+156283_173860869308116_1000005.jpg","size":"mega"}],"streamable":"1"},{"name":"Slipknot","mbid":"a466c2a2-6517-42fb-a160-1087c3bafd9f","match":"0.310848","url":"www.last.fm\/music\/Slipknot","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/7391673.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/7391673.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/7391673.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/7391673.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/7391673\/Slipknot.jpg","size":"mega"}],"streamable":"1"},{"name":"Krisiun","mbid":"9e6458d6-7053-48eb-ac8f-bf6fdf9169c6","match":"0.303816","url":"www.last.fm\/music\/Krisiun","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/355482.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/355482.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/355482.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/355482.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/355482\/Krisiun.jpg","size":"mega"}],"streamable":"1"},{"name":"Hunter","mbid":"2e80c47d-29ca-44e1-bec2-c844d19e411f","match":"0.301649","url":"www.last.fm\/music\/Hunter","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/4990702.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/4990702.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/4990702.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/4990702.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/4990702\/Hunter+2.jpg","size":"mega"}],"streamable":"1"},{"name":"Napalm Death","mbid":"ce7bba8b-026b-4aa6-bddb-f98ed6d595e4","match":"0.295007","url":"www.last.fm\/music\/Napalm+Death","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/69983154.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/69983154.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/69983154.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/69983154.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/69983154\/Napalm+Death+FETO.jpg","size":"mega"}],"streamable":"1"},{"name":"Ozzy Osbourne","mbid":"8aa5b65a-5b3c-4029-92bf-47a544356934","match":"0.291688","url":"www.last.fm\/music\/Ozzy+Osbourne","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/4818557.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/4818557.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/4818557.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/4818557.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/4818557\/Ozzy+Osbourne+ozzz1he4.jpg","size":"mega"}],"streamable":"1"}],"@attr":{"artist":"Slayer"}}},{})
    var scrobbler = new scribble('a','a','a','a')
    scrobbler.GetSimilarArtists(song.artist, function(ret) {
      ret.should.be.a('object').and.have.property('similarartists')
      endTest()
    })
  })
}) // [Similar Artists]

describe('[Artist Events]', function() {
  // 8
  it('should return artist events', function(endTest) {
    // mock request
    nock('http://ws.audioscrobbler.com:80')
        .get('/2.0/?method=artist.getevents&artist=' + song.artist + '&api_key=a&format=json&limit=50')
        .reply(200, {"events":{"event":[{"id":"3296812","title":"Soundwave 2013","artists":{"artist":["Linkin Park","blink-182","The Offspring","Metallica","Paramore","Sum 41","Bullet for My Valentine","A Perfect Circle","Garbage","Stone Sour","Cypress Hill","Billy Talent","Slayer","Killswitch Engage","All Time Low","Motion City Soundtrack","Anthrax","Flogging Molly","Bring Me the Horizon","DragonForce","Mindless Self Indulgence","The Early November","Pierce The Veil","The Vandals","Miss May I","Danko Jones","Sick of It All","The Blackout","Gallows","Tomahawk","Six Feet Under","Chelsea Grin","Fucked Up","Memphis May Fire","The Lawrence Arms","The Sword","The Wonder Years","The Dear Hunter","Madball","Orange Goblin","The Chariot","Lucero","Cancer Bats","Woe, Is Me","Sleeping with Sirens","Of Mice & Men","Ghost","Chunk! No, Captain Chunk!","Polar Bear Club","The Amity Affliction","Deaf Havana","Shai Hulud","Periphery","Red Fang","This is Hell","Sylosis","Fozzy","Such Gold","While She Sleeps","Kingdom of Sorrow","Confession","Vision Of Disorder","SHARKS","Living With Lions","Crossfaith","Duff McKagan's Loaded","Northlane","Cerebral Ballzy","O'Brother","Versus The World","Milestones","Kyuss Lives!"],"headliner":"Linkin Park"},"venue":{"id":"8782466","name":"RNA Showgrounds","location":{"geo:point":{"geo:lat":"-27.451199","geo:long":"153.032043"},"city":"Bowen Hills, Brisbane, Queensland","country":"Australia","street":"600 Gregory Terrace","postalcode":"4006"},"url":"http:\/\/www.last.fm\/venue\/8782466+RNA+Showgrounds","website":"http:\/\/www.rnashowgrounds.com.au","phonenumber":"(13) 1931","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/2622157.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/2622157.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/2622157.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/2622157.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/2622157\/RNA+Showgrounds+rna.jpg","size":"mega"}]},"startDate":"Sat, 23 Feb 2013 06:07:01","description":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/81275525.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/81275525.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/81275525.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/81275525.jpg","size":"extralarge"}],"attendance":"229","reviews":"0","tag":"lastfm:event=3296812","url":"http:\/\/www.last.fm\/festival\/3296812+Soundwave+2013","website":"http:\/\/www.soundwavefestival.com\/","tickets":"\n  ","cancelled":"0"},{"id":"3313215","title":"Soundwave 2013","artists":{"artist":["Linkin Park","blink-182","The Offspring","Metallica","Paramore","Sum 41","Bullet for My Valentine","A Perfect Circle","Garbage","Stone Sour","Cypress Hill","Billy Talent","Slayer","Killswitch Engage","All Time Low","Motion City Soundtrack","Anthrax","Flogging Molly","Bring Me the Horizon","DragonForce","Mindless Self Indulgence","The Early November","Pierce The Veil","The Vandals","Miss May I","Danko Jones","Sick of It All","The Blackout","Gallows","Tomahawk","Six Feet Under","Chelsea Grin","Fucked Up","Memphis May Fire","The Lawrence Arms","The Sword","The Wonder Years","The Dear Hunter","Madball","Orange Goblin","The Chariot","Lucero","Cancer Bats","Woe, Is Me","Sleeping with Sirens","Of Mice & Men","Ghost","Chunk! No, Captain Chunk!","Polar Bear Club","The Amity Affliction","Deaf Havana","Shai Hulud","Periphery","Red Fang","This is Hell","Sylosis","Fozzy","Such Gold","While She Sleeps","Kingdom of Sorrow","Confession","Vision Of Disorder","SHARKS","Living With Lions","Crossfaith","Duff McKagan's Loaded","Northlane","Cerebral Ballzy","O'Brother","Versus The World","Milestones","Kyuss Lives!"],"headliner":"Linkin Park"},"venue":{"id":"8778565","name":"Sydney Showground","location":{"geo:point":{"geo:lat":"-33.84549","geo:long":"151.06915"},"city":"Sydney Olympic Park","country":"Australia","street":"1 Showground Rd","postalcode":"2127"},"url":"http:\/\/www.last.fm\/venue\/8778565+Sydney+Showground","website":"http:\/\/sydneyshowground.com.au\/","phonenumber":"(02) 9704 1111","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/4103807.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/4103807.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/4103807.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/4103807.png","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/4103807\/Sydney+Showground+Picture+4.png","size":"mega"}]},"startDate":"Sun, 24 Feb 2013 11:00:00","description":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/81338867.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/81338867.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/81338867.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/81338867.jpg","size":"extralarge"}],"attendance":"322","reviews":"1","tag":"lastfm:event=3313215","url":"http:\/\/www.last.fm\/festival\/3313215+Soundwave+2013","website":"","tickets":"\n  ","cancelled":"0"},{"id":"3459949","title":"Slayer","artists":{"artist":["Slayer","Anthrax","Kingdom of Sorrow"],"headliner":"Slayer"},"venue":{"id":"8784669","name":"Big Top Luna Park","location":{"geo:point":{"geo:lat":"-33.849815","geo:long":"151.21277"},"city":"Sydney","country":"Australia","street":"1 Olympic Drive, Milsons Point","postalcode":"2061"},"url":"http:\/\/www.last.fm\/venue\/8784669+Big+Top+Luna+Park","website":"http:\/\/www.bigtopsydney.com\/","phonenumber":"1300 BIG TOP","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/4103763.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/4103763.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/4103763.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/4103763.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/4103763\/Big+Top+Luna+Park+LUNAPARKGATE.jpg","size":"mega"}]},"startDate":"Mon, 25 Feb 2013 19:30:00","description":"<div class=\"bbcode\">Ten studio albums, thousands of live shows and nearly three decades into a career that\u2019s made them one of the biggest and most important metal bands in the world, the members of <a href=\"http:\/\/www.last.fm\/music\/Slayer\" class=\"bbcode_artist\">Slayer<\/a> know exactly what kind of music they make\u2014brutal but beautiful, punishing yet precise.<br \/><br \/>This is <a href=\"http:\/\/www.last.fm\/music\/Slayer\" class=\"bbcode_artist\">Slayer<\/a>. Do we really need to go on? The only other thing you need to know is that tickets go on sale Wednesday 19 December, 9am.<\/div>","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/8428755.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/8428755.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/8428755.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/8428755.jpg","size":"extralarge"}],"attendance":"29","reviews":"0","tag":"lastfm:event=3459949","url":"http:\/\/www.last.fm\/event\/3459949+Slayer+at+Big+Top+Luna+Park+on+25+February+2013","website":"http:\/\/www.bigtopsydney.com\/slayer.html","tickets":"\n  ","cancelled":"0","tags":{"tag":["thrash metal","metal","heavy metal"]}},{"id":"3355647","title":"Soundwave 2013","artists":{"artist":["Metallica","Linkin Park","blink-182","The Offspring","Paramore","Sum 41","Bullet for My Valentine","A Perfect Circle","Garbage","Stone Sour","Cypress Hill","Billy Talent","Slayer","Killswitch Engage","All Time Low","Motion City Soundtrack","Anthrax","Flogging Molly","Bring Me the Horizon","DragonForce","Mindless Self Indulgence","The Early November","Pierce The Veil","The Vandals","Miss May I","Danko Jones","Sick of It All","The Blackout","Gallows","Tomahawk","Six Feet Under","Chelsea Grin","Fucked Up","Memphis May Fire","The Lawrence Arms","The Sword","The Wonder Years","The Dear Hunter","Madball","Orange Goblin","The Chariot","Lucero","Cancer Bats","Woe, Is Me","Sleeping with Sirens","Of Mice & Men","Ghost","Chunk! No, Captain Chunk!","Polar Bear Club","The Amity Affliction","Deaf Havana","Shai Hulud","Periphery","Red Fang","This is Hell","Sylosis","Fozzy","Such Gold","While She Sleeps","Kingdom of Sorrow","Confession","Vision Of Disorder","SHARKS","Living With Lions","Crossfaith","Duff McKagan's Loaded","Northlane","Cerebral Ballzy","O'Brother","Versus The World","Milestones","Kyuss Lives!"],"headliner":"Metallica"},"venue":{"id":"8871167","name":"Flemington Racecourse","location":{"geo:point":{"geo:lat":"-37.785771","geo:long":"144.919699"},"city":"Flemington, Melbourne, Victoria","country":"Australia","street":"Leonard Crescent","postalcode":"3031"},"url":"http:\/\/www.last.fm\/venue\/8871167+Flemington+Racecourse","website":"http:\/\/flemington.com.au\/","phonenumber":"","image":[{"#text":"","size":"small"},{"#text":"","size":"medium"},{"#text":"","size":"large"},{"#text":"","size":"extralarge"},{"#text":"","size":"mega"}]},"startDate":"Fri, 01 Mar 2013 01:18:01","description":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/81505587.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/81505587.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/81505587.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/81505587.jpg","size":"extralarge"}],"attendance":"330","reviews":"1","tag":"lastfm:event=3355647","url":"http:\/\/www.last.fm\/festival\/3355647+Soundwave+2013","website":"","tickets":"\n  ","cancelled":"0"},{"id":"3355650","title":"Soundwave 2013","artists":{"artist":["Metallica","Linkin Park","blink-182","The Offspring","Paramore","Sum 41","Bullet for My Valentine","A Perfect Circle","Garbage","Stone Sour","Cypress Hill","Billy Talent","Slayer","Killswitch Engage","All Time Low","Motion City Soundtrack","Anthrax","Flogging Molly","Bring Me the Horizon","DragonForce","Mindless Self Indulgence","The Early November","Pierce The Veil","The Vandals","Miss May I","Danko Jones","Sick of It All","The Blackout","Gallows","Tomahawk","Six Feet Under","Chelsea Grin","Fucked Up","Memphis May Fire","The Lawrence Arms","The Sword","The Wonder Years","The Dear Hunter","Madball","Orange Goblin","The Chariot","Lucero","Cancer Bats","Woe, Is Me","Sleeping with Sirens","Of Mice & Men","Ghost","Chunk! No, Captain Chunk!","Polar Bear Club","The Amity Affliction","Deaf Havana","Shai Hulud","Periphery","Red Fang","This is Hell","Sylosis","Fozzy","Such Gold","While She Sleeps","Kingdom of Sorrow","Confession","Vision Of Disorder","SHARKS","Living With Lions","Crossfaith","Duff McKagan's Loaded","Northlane","Cerebral Ballzy","O'Brother","Versus The World","Milestones","Kyuss Lives!"],"headliner":"Metallica"},"venue":{"id":"8796929","name":"Bonython Park","location":{"geo:point":{"geo:lat":"-34.9333333","geo:long":"138.6"},"city":"Adelaide, South Australia","country":"Australia","street":"Port Road","postalcode":"5000"},"url":"http:\/\/www.last.fm\/venue\/8796929+Bonython+Park","website":"","phonenumber":"08 8203 7203","image":[{"#text":"","size":"small"},{"#text":"","size":"medium"},{"#text":"","size":"large"},{"#text":"","size":"extralarge"},{"#text":"","size":"mega"}]},"startDate":"Sat, 02 Mar 2013 01:37:01","description":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/82379913.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/82379913.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/82379913.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/82379913.jpg","size":"extralarge"}],"attendance":"111","reviews":"0","tag":"lastfm:event=3355650","url":"http:\/\/www.last.fm\/festival\/3355650+Soundwave+2013","website":"http:\/\/www.soundwavefestival.com\/","tickets":"\n  ","cancelled":"0"},{"id":"3355655","title":"Soundwave 2013","artists":{"artist":["Metallica","Linkin Park","blink-182","The Offspring","Paramore","Sum 41","Bullet for My Valentine","A Perfect Circle","Garbage","Stone Sour","Cypress Hill","Billy Talent","Slayer","Killswitch Engage","All Time Low","Motion City Soundtrack","Anthrax","Flogging Molly","Bring Me the Horizon","DragonForce","Mindless Self Indulgence","The Early November","Pierce The Veil","The Vandals","Miss May I","Danko Jones","Sick of It All","The Blackout","Gallows","Tomahawk","Six Feet Under","Chelsea Grin","Fucked Up","Memphis May Fire","The Lawrence Arms","The Sword","The Wonder Years","The Dear Hunter","Madball","Orange Goblin","The Chariot","Lucero","Cancer Bats","Woe, Is Me","Sleeping with Sirens","Of Mice & Men","Ghost","Chunk! No, Captain Chunk!","Polar Bear Club","The Amity Affliction","Deaf Havana","Shai Hulud","Periphery","Red Fang","This is Hell","Sylosis","Fozzy","Such Gold","While She Sleeps","Kingdom of Sorrow","Confession","Vision Of Disorder","SHARKS","Living With Lions","Crossfaith","Duff McKagan's Loaded","Northlane","Cerebral Ballzy","O'Brother","Versus The World","Milestones","Kyuss Lives!"],"headliner":"Metallica"},"venue":{"id":"8778617","name":"Claremont Showgrounds","location":{"geo:point":{"geo:lat":"-31.974168","geo:long":"115.783164"},"city":"Claremont, Western Australia","country":"Australia","street":"Graylands Road","postalcode":"6000"},"url":"http:\/\/www.last.fm\/venue\/8778617+Claremont+Showgrounds","website":"http:\/\/www.claremontshowground.com.au\/","phonenumber":"+(61) (08) 6263 3100","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/16969695.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/16969695.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/16969695.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/16969695.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/16969695\/Claremont+Showgrounds.jpg","size":"mega"}]},"startDate":"Mon, 04 Mar 2013 02:07:01","description":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/83285555.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/83285555.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83285555.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/83285555.jpg","size":"extralarge"}],"attendance":"129","reviews":"0","tag":"lastfm:event=3355655","url":"http:\/\/www.last.fm\/festival\/3355655+Soundwave+2013","website":"","tickets":"\n  ","cancelled":"0"},{"id":"3393106","title":"Impact Festival 2013","artists":{"artist":["Rammstein","Korn","30 Seconds to Mars","Paramore","Slayer","Mastodon","Airbourne","Behemoth","Asking Alexandria","Ghost"],"headliner":"Rammstein"},"venue":{"id":"9014288","name":"Lotnisko Bemowo","location":{"geo:point":{"geo:lat":"52.243869","geo:long":"20.912348"},"city":"Warszawa","country":"Poland","street":"ul. Powsta\u0144c\u00f3w \u015al\u0105skich 125","postalcode":"01-466"},"url":"http:\/\/www.last.fm\/venue\/9014288+Lotnisko+Bemowo","website":"","phonenumber":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/33400737.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/33400737.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/33400737.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/33400737.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/33400737\/Lotnisko+Bemowo+250pxLotniskobemowo.jpg","size":"mega"}]},"startDate":"Tue, 04 Jun 2013 14:00:01","endDate":"Wed, 05 Jun 2013 20:51:01","description":"<div class=\"bbcode\"><a href=\"http:\/\/www.livenation.pl\/event\/346531\/impact-festival-tickets\" rel=\"nofollow\">http:\/\/www.livenation.pl\/event\/346531\/impact-festival-tickets<\/a><br \/><br \/><br \/>4 czerwca: Rammstein, Behemoth, Slayer, Korn, Mastodon, Ghost, Airbourne<br \/>5 czerwca: 30 Seconds To Mars, Asking Alexandria, Paramore<br \/><br \/>Bilety jednodniowe:<br \/>209 z\u0142 - p\u0142yta<br \/>385 z\u0142 - trybuny<br \/>385 z\u0142 - p\u0142yta early entrance (wa\u017cne informacje poni\u017cej)<br \/>880 z\u0142 - VIP<br \/><br \/>Bilety dwudniowe:<br \/>330 z\u0142 - p\u0142yta i osoby niepe\u0142nosprawne<br \/>583 z\u0142 - trybuny<br \/>583 z\u0142 - p\u0142yta early entrance (wa\u017cne informacje poni\u017cej)<br \/>1350 z\u0142 - VIP<br \/><br \/>UWAGA!<br \/>Bilety GAEE (p\u0142yta early entrance o warto\u015bci 385z\u0142) \u2013 uprawniaj\u0105ce do wcze\u015bniejszego wej\u015bcia na teren festiwalu oraz zawieraj\u0105ce voucher upowa\u017cniaj\u0105cy do odbioru w dniu imprezy i na terenie festiwalu pami\u0105tkowego laminatu i koszulki Impact Festival 2013 \u2013 b\u0119d\u0105 dost\u0119pne wy\u0142\u0105cznie via LiveNation.pl<\/div>","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/86553251.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/86553251.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/86553251.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/86553251.jpg","size":"extralarge"}],"attendance":"3417","reviews":"0","tag":"lastfm:event=3393106","url":"http:\/\/www.last.fm\/festival\/3393106+Impact+Festival+2013","website":"http:\/\/www.impactfest.pl\/","tickets":"\n  ","cancelled":"0","tags":{"tag":"metal"}},{"id":"3414905","title":"Sonisphere Festival France 2013","artists":{"artist":["Iron Maiden","Korn","Limp Bizkit","Stone Sour","Mot\u00f6rhead","Slayer","Children of Bodom","Bring Me the Horizon","Mastodon","DragonForce","Amon Amarth","Epica","Airbourne","Behemoth","Sabaton","Ghost","Trust","Dagoba","Headcharger"],"headliner":"Iron Maiden"},"venue":{"id":"10226335","name":"Snowhall Parc","location":{"geo:point":{"geo:lat":"49.248983","geo:long":"6.127925"},"city":"Amn\u00e9ville","country":"France","street":"","postalcode":"57360"},"url":"http:\/\/www.last.fm\/venue\/10226335+Snowhall+Parc","website":"","phonenumber":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/70396480.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/70396480.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/70396480.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/70396480.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/70396480\/Snowhall+Parc+snowhallparc1.jpg","size":"mega"}]},"startDate":"Sat, 08 Jun 2013 17:36:01","endDate":"Sun, 09 Jun 2013 17:36:01","description":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/85298107.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/85298107.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/85298107.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/85298107.jpg","size":"extralarge"}],"attendance":"127","reviews":"0","tag":"lastfm:event=3414905","url":"http:\/\/www.last.fm\/festival\/3414905+Sonisphere+Festival+France+2013","website":"http:\/\/fr.sonisphere.eu","tickets":"\n  ","cancelled":"0","tags":{"tag":"metal"}},{"id":"3393317","title":"Greenfield Festival 2013","artists":{"artist":["Queens of the Stone Age","Bullet for My Valentine","Rammstein","Stone Sour","Nightwish","Slayer","Bad Religion","A Day to Remember","NOFX","Within Temptation","Bring Me the Horizon","The Devil Wears Prada","Parkway Drive","The Gaslight Anthem","The Bouncing Souls","Airbourne","Converge","Every Time I Die","Caliban","Turbonegro","Danko Jones","Boysetsfire","Saltatio Mortis","Graveyard","Jennifer Rostock","Deez Nuts","Red Fang","Eskimo Callboy","Mono Inc."],"headliner":"Queens of the Stone Age"},"venue":{"id":"8786173","name":"Greenfield","location":{"geo:point":{"geo:lat":"46.6833333","geo:long":"7.85"},"city":"Interlaken","country":"Switzerland","street":"","postalcode":""},"url":"http:\/\/www.last.fm\/venue\/8786173+Greenfield","website":"http:\/\/www.greenfieldfestival.ch","phonenumber":"","image":[{"#text":"","size":"small"},{"#text":"","size":"medium"},{"#text":"","size":"large"},{"#text":"","size":"extralarge"},{"#text":"","size":"mega"}]},"startDate":"Thu, 13 Jun 2013 18:19:01","endDate":"Sat, 15 Jun 2013 18:19:01","description":"<div class=\"bbcode\">vom 13. bis 15. Juni 2013 in Interlaken (CH)<br \/><br \/>Neben RAMMSTEIN mit dabei sind QUEENS OF THE STONE AGE, WITHIN TEMPTATION, NOFX, THE GASLIGHT ANTHEM, AIRBOURNE, STONE SOUR, BULLET FOR MY VALENTINE, A DAY TO REMEMBER, BRING ME THE HORIZON, BOYSETSFIRE, DANKO JONES, CALIBAN, EVERY TIME I DIE, THE DEVIL WEARS PRADA, SALTATIO MORTIS, JENNIFER ROSTOCK, THE BOUNCING SOULS, MONO INC. Weitere Bands folgen - stay tuned!<br \/><br \/><a href=\"http:\/\/www.ticketcorner.ch\/greenfield-2013-early-bird-Tickets.html?affiliate=TCS&amp;doc=artistPages%2Ftickets&amp;fun=artist&amp;action=tickets&amp;kuid=430400\" rel=\"nofollow\">http:\/\/www.ticketcorner.ch\/greenfield-2013-early-bird-Tickets.html?affiliate=TCS&amp;doc=artistPages%2Ftickets&amp;fun=artist&amp;action=tickets&amp;kuid=430400<\/a><br \/><br \/>Weitere Infos zum Festival: <a href=\"http:\/\/www.greenfieldfestival.ch\/intro\/index.php\" rel=\"nofollow\">http:\/\/www.greenfieldfestival.ch\/intro\/index.php<\/a><\/div>","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/154940.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/154940.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/154940.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/154940.jpg","size":"extralarge"}],"attendance":"118","reviews":"0","tag":"lastfm:event=3393317","url":"http:\/\/www.last.fm\/festival\/3393317+Greenfield+Festival+2013","website":"https:\/\/www.facebook.com\/events\/466638060025403\/","tickets":"\n  ","cancelled":"0"},{"id":"3393808","title":"Metalfest Open Air 2013 Loreley","artists":{"artist":["Slayer","Children of Bodom","Arch Enemy","Testament","Iced Earth","Accept","Saxon","Paradise Lost","Down","Turisas","J.B.O.","Six Feet Under","Wintersun","Subway To Sally","Equilibrium","Doro","Threshold","TrollfesT","Majesty","Feuerschwanz","Varg","Grailknights","Suicidal Angels","Kissin Dynamite"],"headliner":"Slayer"},"venue":{"id":"8887416","name":"Freilichtb\u00fchne Loreley","location":{"geo:point":{"geo:lat":"50.14229","geo:long":"7.73218"},"city":"Sankt Goarshausen","country":"Germany","street":"Freilichtb\u00fchne Loreley","postalcode":"56346"},"url":"http:\/\/www.last.fm\/venue\/8887416+Freilichtb%C3%BChne+Loreley","website":"http:\/\/www.loreley.biz\/","phonenumber":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/36453921.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/36453921.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/36453921.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/36453921.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/36453921\/Freilichtbhne+Loreley+RockpalastLoreley.jpg","size":"mega"}]},"startDate":"Thu, 20 Jun 2013 10:33:01","endDate":"Sat, 22 Jun 2013 10:33:01","description":"<div class=\"bbcode\">Tickets ab 79\u20ac<\/div>","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/85972377.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/85972377.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/85972377.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/85972377.jpg","size":"extralarge"}],"attendance":"127","reviews":"0","tag":"lastfm:event=3393808","url":"http:\/\/www.last.fm\/festival\/3393808+Metalfest+Open+Air+2013+Loreley","website":"http:\/\/www.metalfest.eu","tickets":"\n  ","cancelled":"0","tags":{"tag":"metal"}},{"id":"3455305","title":"SEE ROCK FESTIVAL","artists":{"artist":["Iron Maiden","Stone Sour","Mot\u00f6rhead","Slayer","Oomph!","Eisbrecher","Ghost","Marrok","Rohstoff"],"headliner":"Iron Maiden"},"venue":{"id":"10289101","name":"Schwarzl Freizeitzentrum","location":{"geo:point":{"geo:lat":"46.981953","geo:long":"15.426083"},"city":"Unterpremst\u00e4tten","country":"Austria","street":"Thalerhofstra\u00dfe 85","postalcode":""},"url":"http:\/\/www.last.fm\/venue\/10289101+Schwarzl+Freizeitzentrum","website":"http:\/\/www.sfz.at","phonenumber":"03135 535770","image":[{"#text":"","size":"small"},{"#text":"","size":"medium"},{"#text":"","size":"large"},{"#text":"","size":"extralarge"},{"#text":"","size":"mega"}]},"startDate":"Fri, 21 Jun 2013 20:44:01","endDate":"Sat, 22 Jun 2013 20:44:01","description":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/84955969.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/84955969.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/84955969.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/84955969.jpg","size":"extralarge"}],"attendance":"127","reviews":"0","tag":"lastfm:event=3455305","url":"http:\/\/www.last.fm\/festival\/3455305+SEE+ROCK+FESTIVAL","website":"http:\/\/www.see-rock.at\/","tickets":"\n  ","cancelled":"0","tags":{"tag":["metal","heavy metal"]}},{"id":"3486150","title":"Hegyalja Fesztiv\u00e1l 2013","artists":{"artist":["Scooter","Slayer","Anti-Flag","The Devil Wears Prada","Coal Chamber","Ensiferum","Every Time I Die","Asking Alexandria","Skindred","Between the Buried and Me","Down","Reso","Dieselboy","Bouncing Souls","Quimby"],"headliner":"Scooter"},"venue":{"id":"10290486","name":"Tisza-part","location":{"geo:point":{"geo:lat":"","geo:long":""},"city":"Tokaj","country":"Hungary","street":"","postalcode":""},"url":"http:\/\/www.last.fm\/venue\/10290486+Tisza-part","website":"","phonenumber":"","image":[{"#text":"","size":"small"},{"#text":"","size":"medium"},{"#text":"","size":"large"},{"#text":"","size":"extralarge"},{"#text":"","size":"mega"}]},"startDate":"Tue, 25 Jun 2013 00:23:01","endDate":"Sat, 29 Jun 2013 00:23:01","description":"<div class=\"bbcode\">A Hegy' az a hely, ahova vissza kell t\u00e9rni, hiszen a tokaji hegyek k\u00f6z\u00f6tt az igazi arcok, az igazi zene, az igazi fesztiv\u00e1l\u00e9lm\u00e9ny v\u00e1r benneteket. A Borsodi Hegyalja Fesztiv\u00e1l az orsz\u00e1g egyetlen olyan n\u00e9gynapos bulija, ahol a fesztiv\u00e1l\u00e9lm\u00e9nyeitek a vil\u00e1g\u00f6r\u00f6ks\u00e9g r\u00e9sz\u00e9v\u00e9 v\u00e1lnak.<br \/><br \/>J\u00fanius 25-30. k\u00f6z\u00f6tt, 14. alkalommal \u00e9lvezhetitek a Tisz\u00e1t, a tokaji borokat, a legizgalmasabb k\u00fclf\u00f6ldi csapatok \u00e9s magyar zen\u00e9szek koncertjeit egyszerre. Itt \u00fcnnepelhetitek meg a vizsgaid\u0151szakot, vagy az \u00e9retts\u00e9gi ut\u00e1n v\u00e9gre felszabadultan tombolhattok kedvenc zenekarotok koncertj\u00e9n, hiszen 2013-ban a Hegy\u2019 nyitja a nyarat.<br \/><br \/>A k\u00fclf\u00f6ldi fell\u00e9p\u0151k mellett Tokajban lesznek a legjobb hazai zen\u00e9szek, a legviccesebb stand-uposok, s\u0151t ez az a fesztiv\u00e1l, ahol a legt\u00f6bb magyar bor\u00e1sszal tal\u00e1lkozhattok. A Borsodi Hegyalja Fesztiv\u00e1lon ak\u00e1r egy\u00fctt s\u00f6r\u00f6zhettek, Furmint-fr\u00f6ccs\u00f6zhettek a kedvenc \u00edr\u00f3itokkal, \u00e9nekeseitekkel, s\u0151t ez az a hely, ahol k\u00e9t koncert k\u00f6z\u00f6tt k\u00f6nnyed\u00e9n munk\u00e1t is tal\u00e1lhattok.<br \/><br \/>T\u00f6bb mint 200 fell\u00e9p\u0151, hajnalig tart\u00f3 p\u00f6rg\u00e9s, evez\u00e9s, sz\u00ednh\u00e1z, strandol\u00e1s, boros pinc\u00e9k, izgalmas besz\u00e9lget\u00e9sek, az orsz\u00e1g legjobb arcaival. 2013-ban a ny\u00e1r egy igazi fesztiv\u00e1llal \u00e9s igazi arcokkal indul Rakamaz-Tokajon.<br \/><br \/><a href=\"https:\/\/www.facebook.com\/hegyaljafesztival\" rel=\"nofollow\">https:\/\/www.facebook.com\/hegyaljafesztival<\/a><br \/><br \/>Jegylink: <a href=\"http:\/\/www.eventim.hu\/hu\/jegyek\/borsodi-hegyalja-fesztival-2013-tokaj-rakamaz-490907\/performance.html\" rel=\"nofollow\">http:\/\/www.eventim.hu\/hu\/jegyek\/borsodi-hegyalja-fesztival-2013-tokaj-rakamaz-490907\/performance.html<\/a><\/div>","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/85888773.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/85888773.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/85888773.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/85888773.jpg","size":"extralarge"}],"attendance":"32","reviews":"0","tag":"lastfm:event=3486150","url":"http:\/\/www.last.fm\/festival\/3486150+Hegyalja+Fesztiv%C3%A1l+2013","website":"http:\/\/www.hegyaljafesztival.hu","tickets":"\n  ","cancelled":"0"},{"id":"3327596","title":"With Full Force XX","artists":{"artist":["Mot\u00f6rhead","Slayer","In Flames","The Devil Wears Prada","Parkway Drive","Coal Chamber","Hatebreed","Amorphis","Paradise Lost","Every Time I Die","Caliban","Asking Alexandria","Between the Buried and Me","Napalm Death","Hellyeah","Down","Sick of It All","Agnostic Front","Terror","Chelsea Grin","H2O","Marduk","The Acacia Strain","Bury Your Dead","Bane","iwrestledabearonce","Knorkator","Your Demise","Naglfar","War From a Harlots Mouth","MAROON","Krisiun","After The Burial","Deez Nuts","Red Fang","Cock Sparrer","Die Kassierer","THE OTHER","Hail Of Bullets","Negura Bunget","Betontod","Ryker's","Mambo Kurt","Hacktivist","Haudegen","Elsterglanz","Buster Shuffle","Unantastbar","AOK","Kali Yuga"],"headliner":"Mot\u00f6rhead"},"venue":{"id":"8785380","name":"Flugplatz Roitzschjora","location":{"geo:point":{"geo:lat":"51.58751","geo:long":"12.485917"},"city":"L\u00f6bnitz","country":"Germany","street":"Am Flugplatz 1","postalcode":"04509"},"url":"http:\/\/www.last.fm\/venue\/8785380+Flugplatz+Roitzschjora","website":"","phonenumber":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/22926311.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/22926311.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/22926311.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/22926311.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/22926311\/Flugplatz+Roitzschjora+With+Full+Force++Main+Stage++F.jpg","size":"mega"}]},"startDate":"Thu, 27 Jun 2013 14:07:01","endDate":"Sun, 30 Jun 2013 14:07:01","description":"<div class=\"bbcode\">Weihnachtskalender:<br \/>01. Hatebreed<br \/>02. Napalm Death<br \/>03. War From A Harlots Mouth<br \/>04. Bury Your Dead<br \/>05. Hail Of Bullets<br \/>06. Ryker's<br \/>07. Deez Nuts<br \/>08. IWRESTLEDABEARONCE<br \/>09. Maroon<br \/>10. Red Fang<br \/>11. Paradise Lost<br \/>12. Cock Sparrer<br \/>13. Mambo Kurt<br \/>14. After The Burial<br \/>15. The Acacia Strain<br \/>16. Marduk<br \/>17. Unantastbar<br \/>18. Amorphis<br \/>19. The Other<br \/>20. Buster Shuffle<br \/>21. Knorkator<br \/>22. Down<br \/>23. Betontod<br \/>24. Slayer<\/div>","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/85412303.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/85412303.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/85412303.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/85412303.jpg","size":"extralarge"}],"attendance":"475","reviews":"0","tag":"lastfm:event=3327596","url":"http:\/\/www.last.fm\/festival\/3327596+With+Full+Force+XX","website":"http:\/\/www.withfullforce.de","tickets":"\n  ","cancelled":"0"},{"id":"3506897","title":"Slayer","artists":{"artist":"Slayer","headliner":"Slayer"},"venue":{"id":"9177489","name":"Plateia Nerou","location":{"geo:point":{"geo:lat":"","geo:long":""},"city":"Athens","country":"Greece","street":"","postalcode":""},"url":"http:\/\/www.last.fm\/venue\/9177489+Plateia+Nerou","website":"","phonenumber":"","image":[{"#text":"","size":"small"},{"#text":"","size":"medium"},{"#text":"","size":"large"},{"#text":"","size":"extralarge"},{"#text":"","size":"mega"}]},"startDate":"Mon, 01 Jul 2013 07:52:01","description":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/8428755.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/8428755.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/8428755.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/8428755.jpg","size":"extralarge"}],"attendance":"5","reviews":"0","tag":"lastfm:event=3506897","url":"http:\/\/www.last.fm\/event\/3506897+Slayer+at+Plateia+Nerou+on+1+July+2013","website":"http:\/\/www.slayer.net\/us\/tour","tickets":"\n  ","cancelled":"0","tags":{"tag":["thrash metal","slayer","metal","heavy metal"]}},{"id":"3484704","title":"Rockstar Mayhem Fesival","artists":{"artist":["Anthrax","Slipknot","Slayer","As I Lay Dying","The Devil Wears Prada","Asking Alexandria","Motorhead","Betraying the Martyrs","Upon A Burning Body","I, The Breather","White Chapel","Dirtfedd"],"headliner":"Anthrax"},"venue":{"id":"8905210","name":"Susquehanna Bank Center","location":{"geo:point":{"geo:lat":"39.94186","geo:long":"-75.129336"},"city":"Camden","country":"United States","street":"1 Harbour Boulevard","postalcode":"08103"},"url":"http:\/\/www.last.fm\/venue\/8905210+Susquehanna+Bank+Center","website":"http:\/\/www.livenation.com\/Susquehanna-Bank-Center-tickets-Camden\/venue\/16465","phonenumber":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/4439338.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/4439338.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/4439338.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/4439338.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/4439338\/Susquehanna+Bank+Center+107_24126_2.jpg","size":"mega"}]},"startDate":"Sat, 27 Jul 2013 20:27:01","description":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/82890667.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/82890667.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/82890667.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/82890667.png","size":"extralarge"}],"attendance":"5","reviews":"0","tag":"lastfm:event=3484704","url":"http:\/\/www.last.fm\/festival\/3484704+Rockstar+Mayhem+Fesival","website":"http:\/\/rockstarmayhemfest.com\/event\/id\/38243","tickets":"\n  ","cancelled":"1","tags":{"tag":["metal","metalcore"]}},{"id":"3373574","title":"\u00d8yafestivalen 2013","artists":{"artist":["Blur","Slayer","The Knife","Of Monsters and Men","James Blake","Solange","Azealia Banks","Mount Kimbie","Alabama Shakes","Electric Wizard","Disclosure","John Talabot","Kvelertak","Haim","Goat","Foxygen","Metz","Angel Haze","John Olav Nilsen & Gjengen","Palma Violets","Black Debbath","Rebekka Karijord","Blood Command","Okkultokrati","T\u00f8nes","Sandra Kolstad","Nils Bech","Monica Heldal","Atlanter","Carmen Villain","Truls","Jupiter Bokonji and Okwess International"],"headliner":"Blur"},"venue":{"id":"8779428","name":"Middelalderparken","location":{"geo:point":{"geo:lat":"59.90425","geo:long":"10.762104"},"city":"Oslo","country":"Norway","street":"","postalcode":""},"url":"http:\/\/www.last.fm\/venue\/8779428+Middelalderparken","website":"","phonenumber":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/3530802.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/3530802.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/3530802.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/3530802.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/3530802\/Middelalderparken+ya.jpg","size":"mega"}]},"startDate":"Tue, 06 Aug 2013 15:19:01","endDate":"Sat, 10 Aug 2013 19:33:01","description":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/24936045.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/24936045.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/24936045.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/24936045.jpg","size":"extralarge"}],"attendance":"119","reviews":"0","tag":"lastfm:event=3373574","url":"http:\/\/www.last.fm\/festival\/3373574+%C3%98yafestivalen+2013","website":"http:\/\/oyafestivalen.com","tickets":"\n  ","cancelled":"0"},{"id":"3486447","title":"Slayer","artists":{"artist":"Slayer","headliner":"Slayer"},"venue":{"id":"8779359","name":"Tivoli Gr\u00f6na Lund","location":{"geo:point":{"geo:lat":"59.323487","geo:long":"18.097393"},"city":"Stockholm","country":"Sweden","street":"Lilla Allm\u00e4nna Gr\u00e4nd 9","postalcode":"115 21"},"url":"http:\/\/www.last.fm\/venue\/8779359+Tivoli+Gr%C3%B6na+Lund","website":"http:\/\/www.gronalund.com","phonenumber":"08-587 501 00","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/583316.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/583316.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/583316.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/583316.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/583316\/Tivoli+Grna+Lund.jpg","size":"mega"}]},"startDate":"Wed, 07 Aug 2013 20:00:00","description":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/8428755.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/8428755.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/8428755.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/8428755.jpg","size":"extralarge"}],"attendance":"22","reviews":"0","tag":"lastfm:event=3486447","url":"http:\/\/www.last.fm\/event\/3486447+Slayer+at+Tivoli+Gr%C3%B6na+Lund+on+7+August+2013","website":"","tickets":"\n  ","cancelled":"0","tags":{"tag":["thrash metal","slayer","metal","heavy metal"]}},{"id":"3359689","title":"Bloodstock Open Air 2013","artists":{"artist":["Slayer","Lamb of God","Anthrax","DevilDriver","Accept","Amorphis","Avantasia","Whitechapel","King Diamond","Kataklysm","Municipal Waste","Dark Funeral","Firewind","Belphegor","Fozzy","Hell","Ex Deo","Negligence"],"headliner":"Slayer"},"venue":{"id":"10290232","name":"Catton Hall","location":{"geo:point":{"geo:lat":"52.7352","geo:long":"-1.695392"},"city":"Catton","country":"United Kingdom","street":"Walton-on-Trent, South Derbyshire","postalcode":"DE12 8LN"},"url":"http:\/\/www.last.fm\/venue\/10290232+Catton+Hall","website":"http:\/\/www.catton-hall.com\/","phonenumber":"01283 716311","image":[{"#text":"","size":"small"},{"#text":"","size":"medium"},{"#text":"","size":"large"},{"#text":"","size":"extralarge"},{"#text":"","size":"mega"}]},"startDate":"Thu, 08 Aug 2013 15:35:01","endDate":"Sun, 11 Aug 2013 15:35:01","description":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/84898317.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/84898317.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/84898317.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/84898317.jpg","size":"extralarge"}],"attendance":"272","reviews":"0","tag":"lastfm:event=3359689","url":"http:\/\/www.last.fm\/festival\/3359689+Bloodstock+Open+Air+2013","website":"http:\/\/www.bloodstock.uk.com\/","tickets":"\n  ","cancelled":"0","tags":{"tag":"metal"}},{"id":"3476935","title":"Jalometalli 2013","artists":{"artist":["Slayer","Orange Goblin","D.R.I.","Tankard","Altair","Hobbs' Angel of Death","Clock Paradox"],"headliner":"Slayer"},"venue":{"id":"8780060","name":"Club Teatria","location":{"geo:point":{"geo:lat":"65.002624","geo:long":"25.470938"},"city":"Oulu","country":"Finland","street":"Rautatienkatu 84","postalcode":"90400"},"url":"http:\/\/www.last.fm\/venue\/8780060+Club+Teatria","website":"http:\/\/www.teatria.com\/","phonenumber":"08 - 330 050","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/3390111.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/3390111.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/3390111.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/3390111.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/3390111\/Club+Teatria++enterance.jpg","size":"mega"}]},"startDate":"Fri, 09 Aug 2013 08:34:01","endDate":"Sat, 10 Aug 2013 08:34:01","description":"<div class=\"bbcode\"><a href=\"http:\/\/www.last.fm\/music\/Slayer\" class=\"bbcode_artist\">Slayer<\/a><br \/><a href=\"http:\/\/www.last.fm\/music\/Hobbs%27+Angel+of+Death\" class=\"bbcode_artist\">Hobbs' Angel of Death<\/a><br \/><a href=\"http:\/\/www.last.fm\/music\/Tankard\" class=\"bbcode_artist\">Tankard<\/a><\/div>","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/8428755.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/8428755.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/8428755.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/8428755.jpg","size":"extralarge"}],"attendance":"45","reviews":"0","tag":"lastfm:event=3476935","url":"http:\/\/www.last.fm\/festival\/3476935+Jalometalli+2013","website":"http:\/\/www.jalometalli.net","tickets":"\n  ","cancelled":"0","tags":{"tag":["thrash metal","metal"]}},{"id":"3365431","title":"A Campingflight to Lowlands Paradise 2013","artists":{"artist":["Bullet for My Valentine","Slayer","The Lumineers","Alabama Shakes","Moderat","Poli\u00e7a","Disclosure","The Menzingers","Kodaline"],"headliner":"Bullet for My Valentine"},"venue":{"id":"8795559","name":"Evenemententerrein Walibi World","location":{"geo:point":{"geo:lat":"52.438945","geo:long":"5.771056"},"city":"Biddinghuizen","country":"Netherlands","street":"Spijkweg 30","postalcode":"8256 RJ"},"url":"http:\/\/www.last.fm\/venue\/8795559+Evenemententerrein+Walibi+World","website":"http:\/\/www.walibiworld.nl","phonenumber":"0321-32-9999","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/34256537.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/34256537.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/34256537.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/34256537.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/34256537\/Evenemententerrein+Walibi+World+ST_KP4FLHSLOWLANDS_841500b.jpg","size":"mega"}]},"startDate":"Fri, 16 Aug 2013 12:13:01","endDate":"Sun, 18 Aug 2013 12:13:01","description":"<div class=\"bbcode\">Bron datum: <a href=\"http:\/\/www.lowlove.nl\/nieuws\/voor-in-de-agenda-lowlands-2013-op-16-17-en-18-augustus\/\" rel=\"nofollow\">http:\/\/www.lowlove.nl\/nieuws\/voor-in-de-agenda-lowlands-2013-op-16-17-en-18-augustus\/<\/a><br \/><br \/>+NJO Reich Ensemble<\/div>","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/86467927.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/86467927.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/86467927.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/86467927.png","size":"extralarge"}],"attendance":"352","reviews":"0","tag":"lastfm:event=3365431","url":"http:\/\/www.last.fm\/festival\/3365431+A+Campingflight+to+Lowlands+Paradise+2013","website":"http:\/\/lowlands.nl","tickets":"\n  ","cancelled":"0"},{"id":"3504269","title":"Elbriot","artists":{"artist":["Slayer","Bullet for My Valentine","Killswitch Engage","Trivium","As I Lay Dying","Anthrax","Fear Factory","Whitechapel","Betontod"],"headliner":"Slayer"},"venue":{"id":"9126009","name":"Gro\u00dfmarkt","location":{"geo:point":{"geo:lat":"","geo:long":""},"city":"Hamburg","country":"Germany","street":"","postalcode":""},"url":"http:\/\/www.last.fm\/venue\/9126009+Gro%C3%9Fmarkt","website":"","phonenumber":"","image":[{"#text":"","size":"small"},{"#text":"","size":"medium"},{"#text":"","size":"large"},{"#text":"","size":"extralarge"},{"#text":"","size":"mega"}]},"startDate":"Sat, 17 Aug 2013 13:05:01","description":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/8428755.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/8428755.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/8428755.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/8428755.jpg","size":"extralarge"}],"attendance":"34","reviews":"0","tag":"lastfm:event=3504269","url":"http:\/\/www.last.fm\/festival\/3504269+Elbriot","website":"http:\/\/www.elbriot.de","tickets":"\n  ","cancelled":"0","tags":{"tag":["metal","metalcore"]}},{"id":"3504594","title":"X Open Air Rockfest","artists":{"artist":["Bullet for My Valentine","Slayer","Killswitch Engage","Trivium","As I Lay Dying","Anthrax","Fear Factory","DevilDriver","Whitechapel","While She Sleeps"],"headliner":"Bullet for My Valentine"},"venue":{"id":"10276664","name":"Club X","location":{"geo:point":{"geo:lat":"","geo:long":""},"city":"Herford","country":"Germany","street":"","postalcode":""},"url":"http:\/\/www.last.fm\/venue\/10276664+Club+X","website":"","phonenumber":"","image":[{"#text":"","size":"small"},{"#text":"","size":"medium"},{"#text":"","size":"large"},{"#text":"","size":"extralarge"},{"#text":"","size":"mega"}]},"startDate":"Sun, 18 Aug 2013 19:50:01","description":"<div class=\"bbcode\">Beginn 11:00 Uhr<\/div>","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/45613985.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/45613985.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/45613985.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/45613985.jpg","size":"extralarge"}],"attendance":"15","reviews":"0","tag":"lastfm:event=3504594","url":"http:\/\/www.last.fm\/festival\/3504594+X+Open+Air+Rockfest","website":"http:\/\/www.erwin-event.de\/event_detail\/?em_mask=detail&em_leo=761817&ref=L2V2ZW50X3Jlc3VsdC9pbmRleC5waHA\/ZW1fbWFzaz1yZXN1bHQmd2hlcmU9SGVyZm9yZCZyZWdpb249NSZkYXRlX3R5cGU9NyZwYWdlPTI=","tickets":"\n  ","cancelled":"0","tags":{"tag":["metal","metalcore"]}},{"id":"3202057","title":"Rock in Rio V","artists":{"artist":["Muse","Beyonc\u00e9","Nickelback","Metallica","John Mayer","Iron Maiden","Alice in Chains","Avenged Sevenfold","Slayer","Ben Harper","Sepultura","George Benson","Bruce Springsteen & The E Street Band","Ghost","Ivan Lins","Les Tambours du Bronx"],"headliner":"Muse"},"venue":{"id":"8803325","name":"Cidade do Rock","location":{"geo:point":{"geo:lat":"-22.9","geo:long":"-43.2333333"},"city":"Rio de Janeiro","country":"Brazil","street":"Avenida Salvador Allende, s\/n\u00b0- Barra da Tijuca","postalcode":"22780-160"},"url":"http:\/\/www.last.fm\/venue\/8803325+Cidade+do+Rock","website":"","phonenumber":"","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/83911811.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/83911811.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83911811.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/83911811.jpg","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/500\/83911811\/Cidade+do+Rock.jpg","size":"mega"}]},"startDate":"Fri, 13 Sep 2013 02:26:01","endDate":"Sun, 22 Sep 2013 02:26:01","description":"<div class=\"bbcode\">Ingressos para o Rock in Rio<br \/><br \/>Desde o dia 1\u00ba de fevereiro de 2013, o p\u00fablico que se adiantou e adquiriu o Rock in Rio Card \u2014 cart\u00e3o que garantiu a presen\u00e7a antecipada do p\u00fablico no festival antes mesmo do an\u00fancio das bandas e atra\u00e7\u00f5es \u2014 pode escolher o dia de festival ao qual deseja associar o seu cart\u00e3o. A escolha da data pode ser feita at\u00e9 o dia 1\u00ba de abril atrav\u00e9s do site <a href=\"http:\/\/www.rockinrio.ingresso.com\" rel=\"nofollow\">www.rockinrio.ingresso.com<\/a>.  As vendas do Rock in Rio Card foram realizadas em outubro de 2012 e os 80 mil ingressos disponibilizados esgotaram em apenas 52 minutos, um recorde absoluto.<br \/><br \/>Marque na agenda: a venda oficial de ingressos para o Rock in Rio est\u00e1 prevista para abril de 2013. O valor do ingresso \u00e9 R$260 (inteira) e R$130 (meia-entrada).<br \/><br \/>---<br \/>Line-Up<br \/><br \/>Palco Mundo:<br \/>13\/09: Beyonc\u00e9.<br \/>14\/09: Muse.<br \/>15\/09: John Mayer, Bruce Springsteen &amp; The E Street Band.<br \/>19\/09: Metallica, Alice in Chains, Ghost, Sepultura + Les Tambours du Bronx.<br \/>20\/09: Nickelback.<br \/>21\/09: <br \/>22\/09: Iron Maiden, Avenged Sevenfold, Slayer.<br \/><br \/>Palco Sunset:<br \/>13\/09: <br \/>14\/09: <br \/>15\/09: Ivan Lins + George Benson.<br \/>19\/09: <br \/>20\/09: Ben Harper.<br \/>21\/09: <br \/>22\/09: <br \/><br \/>---<br \/>Boatos (Especula\u00e7\u00e3o, Sondados, Em Negocia\u00e7\u00e3o, Faltando Assinar, ETC):<br \/>Adele, AC\/DC, Radiohead, Marky Ramone\u2019s Blitzkrieg, , Eminem, Gogol Bordello, Rob Zombie, Bullet For My Valentine.<br \/>Fonte 1: <a href=\"https:\/\/twitter.com\/jnflesch\" rel=\"nofollow\">https:\/\/twitter.com\/jnflesch<\/a><br \/>Fonte 2: <a href=\"http:\/\/www.destakjornal.com.br\" rel=\"nofollow\">http:\/\/www.destakjornal.com.br<\/a><br \/>---<\/div>","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34\/75059922.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64\/75059922.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/75059922.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/252\/75059922.jpg","size":"extralarge"}],"attendance":"1444","reviews":"0","tag":"lastfm:event=3202057","url":"http:\/\/www.last.fm\/festival\/3202057+Rock+in+Rio+V","website":"http:\/\/www.rockinrio.com.br\/","tickets":"\n  ","cancelled":"0","tags":{"tag":"rock"}}],"@attr":{"artist":"Slayer","festivalsonly":"0","page":"1","perPage":"50","totalPages":"1","total":"23"}}},{})
    var scrobbler = new scribble('a','a','a','a')
    scrobbler.GetArtistEvents(song.artist, function(ret) {
      ret.should.be.a('object').and.have.property('events')
      endTest()
    })
  })
}) // [Artist Events]

describe('[Artist Top Albums]', function() {
  // 9
  it('should return artist top albums', function(endTest) {
    // mock request
    nock('http://ws.audioscrobbler.com:80')
        .get('/2.0/?method=artist.gettopalbums&artist=' + song.artist + '&api_key=a&format=json&limit=50')
        .reply(200, {"topalbums":{"album":[{"name":"Reign in Blood","playcount":"721789","mbid":"04714089-0f2c-3304-a56d-b1e207f5ee2f","url":"http:\/\/www.last.fm\/music\/Slayer\/Reign+in+Blood","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/46065199.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/46065199.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/46065199.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/46065199.png","size":"extralarge"}],"@attr":{"rank":"1"}},{"name":"Seasons in the Abyss","playcount":"345523","mbid":"13eeaa46-e074-3c66-bf2e-474d8c96eeab","url":"http:\/\/www.last.fm\/music\/Slayer\/Seasons+in+the+Abyss","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47615451.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47615451.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47615451.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47615451.png","size":"extralarge"}],"@attr":{"rank":"2"}},{"name":"South of Heaven","playcount":"318306","mbid":"1a326442-3bd0-4094-9ec6-cac9a2f9082d","url":"http:\/\/www.last.fm\/music\/Slayer\/South+of+Heaven","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47464609.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47464609.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47464609.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47464609.png","size":"extralarge"}],"@attr":{"rank":"3"}},{"name":"God Hates Us All","playcount":"355384","mbid":"1c233932-faeb-48a7-96a5-a79871522e40","url":"http:\/\/www.last.fm\/music\/Slayer\/God+Hates+Us+All","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/57560087.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/57560087.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/57560087.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/57560087.png","size":"extralarge"}],"@attr":{"rank":"4"}},{"name":"Show No Mercy","playcount":"233207","mbid":"15baf22c-46ed-4ad9-81b8-2d5ca4eb9347","url":"http:\/\/www.last.fm\/music\/Slayer\/Show+No+Mercy","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/69526808.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/69526808.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/69526808.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/69526808.png","size":"extralarge"}],"@attr":{"rank":"5"}},{"name":"Christ Illusion","playcount":"190682","mbid":"24a80dbb-a639-48e5-80be-9fbf07d4f2bd","url":"http:\/\/www.last.fm\/music\/Slayer\/Christ+Illusion","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/23608071.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/23608071.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/23608071.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/23608071.jpg","size":"extralarge"}],"@attr":{"rank":"6"}},{"name":"World Painted Blood","playcount":"227198","mbid":"99241fe7-0e0b-4544-9db4-0071e863d194","url":"http:\/\/www.last.fm\/music\/Slayer\/World+Painted+Blood","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/57559057.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/57559057.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/57559057.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/57559057.png","size":"extralarge"}],"@attr":{"rank":"7"}},{"name":"Hell Awaits","playcount":"122806","mbid":"2d1e8fd4-1e03-4bbb-a8e8-f65acbba9270","url":"http:\/\/www.last.fm\/music\/Slayer\/Hell+Awaits","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/69450894.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/69450894.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/69450894.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/69450894.png","size":"extralarge"}],"@attr":{"rank":"8"}},{"name":"Diabolus In Musica","playcount":"145327","mbid":"0852d0f2-4d36-3d6c-b72f-3a15a878ab96","url":"http:\/\/www.last.fm\/music\/Slayer\/Diabolus+In+Musica","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/79002941.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/79002941.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/79002941.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/79002941.jpg","size":"extralarge"}],"@attr":{"rank":"9"}},{"name":"Divine Intervention","playcount":"122880","mbid":"59623cbf-08ed-4e64-8a35-5b0cb08b82f4","url":"http:\/\/www.last.fm\/music\/Slayer\/Divine+Intervention","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/57561495.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/57561495.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/57561495.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/57561495.png","size":"extralarge"}],"@attr":{"rank":"10"}},{"name":"Undisputed Attitude","playcount":"128192","mbid":"673e83e0-a615-45cf-a4c1-00bdf6461c80","url":"http:\/\/www.last.fm\/music\/Slayer\/Undisputed+Attitude","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/71032574.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/71032574.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/71032574.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/71032574.png","size":"extralarge"}],"@attr":{"rank":"11"}},{"name":"Live Undead","playcount":"32007","mbid":"301e6dfb-93f2-4827-ae1d-6dca5795566e","url":"http:\/\/www.last.fm\/music\/Slayer\/Live+Undead","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/54966909.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/54966909.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/54966909.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/54966909.png","size":"extralarge"}],"@attr":{"rank":"12"}},{"name":"Haunting the Chapel","playcount":"31231","mbid":"1d9e24a8-cc57-3516-9d9a-8a22f5530b46","url":"http:\/\/www.last.fm\/music\/Slayer\/Haunting+the+Chapel","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/77650138.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/77650138.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/77650138.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/77650138.png","size":"extralarge"}],"@attr":{"rank":"13"}},{"name":"Soundtrack To The Apocalypse","playcount":"37465","mbid":"030bef29-8664-47ff-8903-89f30ef510b5","url":"http:\/\/www.last.fm\/music\/Slayer\/Soundtrack+To+The+Apocalypse","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/79002753.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/79002753.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/79002753.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/79002753.jpg","size":"extralarge"}],"@attr":{"rank":"14"}},{"name":"Soundtrack to the Apocalypse (disc 2)","playcount":"6686","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Soundtrack+to+the+Apocalypse+(disc+2)","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/83419183.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/83419183.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83419183.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/83419183.jpg","size":"extralarge"}],"@attr":{"rank":"15"}},{"name":"God Hates Us All (unmastered advance)","playcount":"2778","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/God+Hates+Us+All+(unmastered+advance)","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19778897.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19778897.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19778897.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19778897.jpg","size":"extralarge"}],"@attr":{"rank":"16"}},{"name":"Live: Decade Of Aggression","playcount":"9111","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Live:+Decade+Of+Aggression","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/images.amazon.com\/images\/P\/B000009QPK.01._SCMZZZZZZZ_.jpg","size":"small"},{"#text":"http:\/\/images.amazon.com\/images\/P\/B000009QPK.01._SCMZZZZZZZ_.jpg","size":"medium"},{"#text":"http:\/\/images.amazon.com\/images\/P\/B000009QPK.01._SCMZZZZZZZ_.jpg","size":"large"},{"#text":"http:\/\/images.amazon.com\/images\/P\/B000009QPK.01._SCMZZZZZZZ_.jpg","size":"extralarge"}],"@attr":{"rank":"17"}},{"name":"Soundtrack to the Apocalypse (disc 5: Live at The Grove in Anaheim, CA 5\/2\/02)","playcount":"4336","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Soundtrack+to+the+Apocalypse+(disc+5:+Live+at+The+Grove+in+Anaheim,+CA+5%2F2%2F02)","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19769009.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19769009.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19769009.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19769009.jpg","size":"extralarge"}],"@attr":{"rank":"18"}},{"name":"Live Intrusion","playcount":"2201","mbid":"a200e6c7-7220-4908-8f25-1d84994f92b1","url":"http:\/\/www.last.fm\/music\/Slayer\/Live+Intrusion","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19778367.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19778367.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19778367.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19778367.jpg","size":"extralarge"}],"@attr":{"rank":"19"}},{"name":"Serenity in Murder","playcount":"1213","mbid":"2e414912-3d7b-4446-9579-5f4e479d540e","url":"http:\/\/www.last.fm\/music\/Slayer\/Serenity+in+Murder","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19769349.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19769349.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19769349.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19769349.jpg","size":"extralarge"}],"@attr":{"rank":"20"}},{"name":"Decade of Aggression: Live","playcount":"2676","mbid":"2988c5e1-cc91-3aa3-8035-14d9174086ab","url":"http:\/\/www.last.fm\/music\/Slayer\/Decade+of+Aggression:+Live","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19786611.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19786611.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19786611.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19786611.jpg","size":"extralarge"}],"@attr":{"rank":"21"}},{"name":"Live Decade of Agression","playcount":"614","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Live+Decade+of+Agression","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/images.amazon.com\/images\/P\/B000062YAY.03.MZZZZZZZ.jpg","size":"small"},{"#text":"http:\/\/images.amazon.com\/images\/P\/B000062YAY.03.MZZZZZZZ.jpg","size":"medium"},{"#text":"http:\/\/images.amazon.com\/images\/P\/B000062YAY.03.MZZZZZZZ.jpg","size":"large"},{"#text":"http:\/\/images.amazon.com\/images\/P\/B000062YAY.03.MZZZZZZZ.jpg","size":"extralarge"}],"@attr":{"rank":"22"}},{"name":"Live Undead\/Haunting The Chapel","playcount":"1034","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Live+Undead%2FHaunting+The+Chapel","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/46135801.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/46135801.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/46135801.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/46135801.jpg","size":"extralarge"}],"@attr":{"rank":"23"}},{"name":"Stain of Mind","playcount":"896","mbid":"1b1c3d04-ff0e-40e0-b755-5cd42f24c687","url":"http:\/\/www.last.fm\/music\/Slayer\/Stain+of+Mind","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19988477.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19988477.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19988477.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19988477.jpg","size":"extralarge"}],"@attr":{"rank":"24"}},{"name":"Live  Decade Of Aggression","playcount":"486","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Live++Decade+Of+Aggression","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/56902811.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/56902811.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/56902811.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/56902811.jpg","size":"extralarge"}],"@attr":{"rank":"25"}},{"name":"Undisputed Attitude\/South Of Heaven","playcount":"455","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Undisputed+Attitude%2FSouth+Of+Heaven","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/63120487.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/63120487.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/63120487.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/63120487.jpg","size":"extralarge"}],"@attr":{"rank":"26"}},{"name":"Still Reigning","playcount":"1326","mbid":"1f9dc2d9-f988-4173-a0c9-2fc5887cd648","url":"http:\/\/www.last.fm\/music\/Slayer\/Still+Reigning","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/79043637.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/79043637.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/79043637.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/79043637.jpg","size":"extralarge"}],"@attr":{"rank":"27"}},{"name":"Eternal Pyre","playcount":"310","mbid":"219e44d5-5c05-498a-829e-13c253b6f360","url":"http:\/\/www.last.fm\/music\/Slayer\/Eternal+Pyre","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/79002991.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/79002991.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/79002991.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/79002991.jpg","size":"extralarge"}],"@attr":{"rank":"28"}},{"name":"Show No Mercy \/ Haunting the Chapel","playcount":"695","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Show+No+Mercy+%2F+Haunting+the+Chapel","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/82629013.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/82629013.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/82629013.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/82629013.png","size":"extralarge"}],"@attr":{"rank":"29"}},{"name":"Soundtrack to the Apocalypse (disc 1)","playcount":"870","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Soundtrack+to+the+Apocalypse+(disc+1)","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/83419125.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/83419125.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83419125.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/83419125.jpg","size":"extralarge"}],"@attr":{"rank":"30"}},{"name":"Live:  Decade Of Aggression","playcount":"897","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Live:++Decade+Of+Aggression","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/83561879.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/83561879.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83561879.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/83561879.jpg","size":"extralarge"}],"@attr":{"rank":"31"}},{"name":"Decade of Aggression: Live (disc 1)","playcount":"512","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Decade+of+Aggression:+Live+(disc+1)","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19786871.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19786871.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19786871.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19786871.jpg","size":"extralarge"}],"@attr":{"rank":"32"}},{"name":"Decade of Aggression: Live (disc 2)","playcount":"369","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Decade+of+Aggression:+Live+(disc+2)","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/7009925.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/7009925.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/7009925.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/7009925.jpg","size":"extralarge"}],"@attr":{"rank":"33"}},{"name":"Eyes Of The Insane","playcount":"114","mbid":"e2320c78-0840-481f-80fd-1a9da7019ba7","url":"http:\/\/www.last.fm\/music\/Slayer\/Eyes+Of+The+Insane","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/10438099.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/10438099.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/10438099.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/10438099.jpg","size":"extralarge"}],"@attr":{"rank":"34"}},{"name":"Stain of Mind + 4 Live Tracks","playcount":"229","mbid":"4cbdbe55-ac1b-4500-92a6-5de24696eb9d","url":"http:\/\/www.last.fm\/music\/Slayer\/Stain+of+Mind+%252B+4+Live+Tracks","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19781287.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19781287.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19781287.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19781287.jpg","size":"extralarge"}],"@attr":{"rank":"35"}},{"name":"Spirits in Black","playcount":"51","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Spirits+in+Black","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19779355.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19779355.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19779355.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19779355.jpg","size":"extralarge"}],"@attr":{"rank":"36"}},{"name":"Soundtrack to the Apocalypse (disc 3: Shit You've Never Heard)","playcount":"232","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Soundtrack+to+the+Apocalypse+(disc+3:+Shit+You%27ve+Never+Heard)","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/83419151.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/83419151.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83419151.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/83419151.jpg","size":"extralarge"}],"@attr":{"rank":"37"}},{"name":"Soundtrack to the Apocalypse (disc 4: Live DVD)","playcount":"16","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Soundtrack+to+the+Apocalypse+(disc+4:+Live+DVD)","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19871277.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19871277.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19871277.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19871277.jpg","size":"extralarge"}],"@attr":{"rank":"38"}},{"name":"Church of Slaytan","playcount":"92","mbid":"cde06d38-fbb2-4772-b4be-c2036b7d3070","url":"http:\/\/www.last.fm\/music\/Slayer\/Church+of+Slaytan","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19867109.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19867109.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19867109.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19867109.jpg","size":"extralarge"}],"@attr":{"rank":"39"}},{"name":"Ubernoise: The Interview","playcount":"135","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Ubernoise:+The+Interview","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19783551.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19783551.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19783551.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19783551.png","size":"extralarge"}],"@attr":{"rank":"40"}},{"name":"Diabolus in Musica (disc 2)","playcount":"83","mbid":"7a3644dc-644f-44a0-88b2-6a53215855d5","url":"http:\/\/www.last.fm\/music\/Slayer\/Diabolus+in+Musica+(disc+2)","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19869383.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19869383.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19869383.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19869383.jpg","size":"extralarge"}],"@attr":{"rank":"41"}},{"name":"Seasons in the Abyss \/ Reign in Blood","playcount":"89","mbid":"4f4bca2a-b1d4-42fb-a01c-304c8fc389c8","url":"http:\/\/www.last.fm\/music\/Slayer\/Seasons+in+the+Abyss+%2F+Reign+in+Blood","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19870705.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19870705.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19870705.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19870705.jpg","size":"extralarge"}],"@attr":{"rank":"42"}},{"name":"Darkness of Christ","playcount":"74","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Darkness+of+Christ","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19785239.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19785239.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19785239.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19785239.png","size":"extralarge"}],"@attr":{"rank":"43"}},{"name":"Wardance","playcount":"29","mbid":"2deff90f-86cb-4f2d-b7c7-c1f5293c346f","url":"http:\/\/www.last.fm\/music\/Slayer\/Wardance","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19780281.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19780281.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19780281.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19780281.jpg","size":"extralarge"}],"@attr":{"rank":"44"}},{"name":"Back in the Game","playcount":"42","mbid":"abd8ce81-b51b-460a-a6e2-17ce38eada89","url":"http:\/\/www.last.fm\/music\/Slayer\/Back+in+the+Game","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19784703.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19784703.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19784703.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19784703.png","size":"extralarge"}],"@attr":{"rank":"45"}},{"name":"War at the Warfield (live 2003)","playcount":"23","mbid":"818c26ff-173b-4574-b82b-532d06094449","url":"http:\/\/www.last.fm\/music\/Slayer\/War+at+the+Warfield+(live+2003)","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/3837052.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/3837052.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/3837052.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/3837052.jpg","size":"extralarge"}],"@attr":{"rank":"46"}},{"name":"Praise Of Death","playcount":"8","mbid":"1beee599-7231-4a4f-94bc-4ca37e6844e7","url":"http:\/\/www.last.fm\/music\/Slayer\/Praise+Of+Death","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19783245.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19783245.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19783245.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19783245.png","size":"extralarge"}],"@attr":{"rank":"47"}},{"name":"Live At The Roskilde Festival June 1998 (25-6-1998)","playcount":"2","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Live+At+The+Roskilde+Festival+June+1998+(25-6-1998)","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19784347.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19784347.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19784347.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19784347.jpg","size":"extralarge"}],"@attr":{"rank":"48"}},{"name":"[non-album tracks]","playcount":"2","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/%5Bnon-album+tracks%5D","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/cdn.last.fm\/flatness\/catalogue\/noimage\/2\/default_album_medium.png","size":"small"},{"#text":"http:\/\/cdn.last.fm\/flatness\/catalogue\/noimage\/2\/default_album_medium.png","size":"medium"},{"#text":"http:\/\/cdn.last.fm\/flatness\/catalogue\/noimage\/2\/default_album_medium.png","size":"large"},{"#text":"http:\/\/cdn.last.fm\/flatness\/catalogue\/noimage\/2\/default_album_medium.png","size":"extralarge"}],"@attr":{"rank":"49"}},{"name":"Jihad","playcount":"1","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Jihad","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/37068201.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/37068201.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/37068201.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/37068201.jpg","size":"extralarge"}],"@attr":{"rank":"50"}}],"@attr":{"artist":"Slayer","page":"1","perPage":"50","totalPages":"2","total":"73"}}},{})
    var scrobbler = new scribble('a','a','a','a')
    scrobbler.GetArtistTopAlbums(song.artist, function(ret) {
      ret.should.be.a('object').and.have.property('topalbums')
      endTest()
    })
  })
}) // [Artist Top Albums]

describe('[Artist Top Tracks]', function() {
  // 10
  it('should return artist top tracks', function(endTest) {
    // mock request
    nock('http://ws.audioscrobbler.com:80')
        .get('/2.0/?method=artist.gettoptracks&artist=' + song.artist + '&api_key=a&format=json&limit=50')
        .reply(200, {"toptracks":{"track":[{"name":"Raining Blood","duration":"253","playcount":"166127","listeners":"55447","mbid":"46d8d34c-51cd-4765-9add-8e38fc6620b2","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Raining+Blood","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/46065199.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/46065199.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/46065199.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/46065199.png","size":"extralarge"}],"@attr":{"rank":"1"}},{"name":"Angel Of Death","duration":"291","playcount":"149633","listeners":"49385","mbid":"c5f536d4-3618-472e-b4b4-f884609416bc","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Angel+Of+Death","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/46065199.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/46065199.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/46065199.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/46065199.png","size":"extralarge"}],"@attr":{"rank":"2"}},{"name":"South Of Heaven","duration":"286","playcount":"79832","listeners":"27859","mbid":"3830ff44-062c-43cb-a2da-7be37588b481","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/South+Of+Heaven","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47464609.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47464609.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47464609.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47464609.png","size":"extralarge"}],"@attr":{"rank":"3"}},{"name":"Piece By Piece","duration":"122","playcount":"68467","listeners":"27831","mbid":"6751bbbd-a4fe-4096-bf16-6d50b6318cd5","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Piece+By+Piece","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/46065199.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/46065199.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/46065199.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/46065199.png","size":"extralarge"}],"@attr":{"rank":"4"}},{"name":"Jesus Saves","duration":"174","playcount":"68552","listeners":"26868","mbid":"06f97ea9-a5f9-4d1d-beb3-35246d3551bc","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Jesus+Saves","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/46065199.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/46065199.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/46065199.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/46065199.png","size":"extralarge"}],"@attr":{"rank":"5"}},{"name":"Postmortem","duration":"207","playcount":"70500","listeners":"26479","mbid":"8d72c0b7-e863-4de8-b6fd-942784fd3e30","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Postmortem","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/46065199.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/46065199.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/46065199.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/46065199.png","size":"extralarge"}],"@attr":{"rank":"6"}},{"name":"Altar Of Sacrifice","duration":"170","playcount":"66058","listeners":"25907","mbid":"ae0a6566-87af-41ad-97f3-5c4309b87f2b","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Altar+Of+Sacrifice","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/46065199.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/46065199.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/46065199.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/46065199.png","size":"extralarge"}],"@attr":{"rank":"7"}},{"name":"Necrophobic","duration":"100","playcount":"64855","listeners":"25810","mbid":"05743840-2335-4629-aca1-1d982cf6b119","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Necrophobic","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/46065199.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/46065199.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/46065199.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/46065199.png","size":"extralarge"}],"@attr":{"rank":"8"}},{"name":"Criminally Insane","duration":"143","playcount":"58284","listeners":"24394","mbid":"79eb900e-9391-4785-a7cc-5f937d177690","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Criminally+Insane","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/46065199.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/46065199.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/46065199.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/46065199.png","size":"extralarge"}],"@attr":{"rank":"9"}},{"name":"Reborn","duration":"131","playcount":"53100","listeners":"22740","mbid":"1dee34f4-d407-4b50-b8d8-d9df7dbb2b11","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Reborn","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/46065199.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/46065199.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/46065199.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/46065199.png","size":"extralarge"}],"@attr":{"rank":"10"}},{"name":"War Ensemble","duration":"292","playcount":"62103","listeners":"22205","mbid":"88165991-9ab5-435c-9a92-374d56a92e61","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/War+Ensemble","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47615451.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47615451.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47615451.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47615451.png","size":"extralarge"}],"@attr":{"rank":"11"}},{"name":"Seasons In The Abyss","duration":"387","playcount":"62322","listeners":"21672","mbid":"70830ad0-8164-4f7f-8fb8-cf479156efa3","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Seasons+In+The+Abyss","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47615451.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47615451.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47615451.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47615451.png","size":"extralarge"}],"@attr":{"rank":"12"}},{"name":"Epidemic","duration":"143","playcount":"51673","listeners":"21670","mbid":"0713f099-db8c-472e-84ad-04a1191a4b1b","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Epidemic","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/46065199.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/46065199.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/46065199.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/46065199.png","size":"extralarge"}],"@attr":{"rank":"13"}},{"name":"Dead Skin Mask","duration":"316","playcount":"58236","listeners":"20562","mbid":"5da2c4be-b11a-4991-aec4-3da6cd50b459","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Dead+Skin+Mask","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47615451.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47615451.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47615451.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47615451.png","size":"extralarge"}],"@attr":{"rank":"14"}},{"name":"Disciple","duration":"214","playcount":"64253","listeners":"20557","mbid":"4899b9b5-d45c-405e-b9b4-970416ad9aa7","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Disciple","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/83419183.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/83419183.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83419183.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/83419183.jpg","size":"extralarge"}],"@attr":{"rank":"15"}},{"name":"Bloodline","duration":"200","playcount":"52162","listeners":"16943","mbid":"7de08a02-4682-4d8c-b38e-d16f3c8fecdc","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Bloodline","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19778897.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19778897.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19778897.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19778897.jpg","size":"extralarge"}],"@attr":{"rank":"16"}},{"name":"Mandatory Suicide","duration":"245","playcount":"44907","listeners":"16844","mbid":"24b4ac1b-a99e-4e3f-8b42-1840b09b8f80","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Mandatory+Suicide","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47464609.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47464609.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47464609.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47464609.png","size":"extralarge"}],"@attr":{"rank":"17"}},{"name":"Blood Red","duration":"168","playcount":"39176","listeners":"16105","mbid":"10341f1a-4fee-43d9-870d-e64d3a8c0eb7","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Blood+Red","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47615451.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47615451.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47615451.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47615451.png","size":"extralarge"}],"@attr":{"rank":"18"}},{"name":"Silent Scream","duration":"230","playcount":"40110","listeners":"16057","mbid":"13892497-aec6-4cba-9143-c733deab4ce0","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Silent+Scream","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47464609.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47464609.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47464609.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47464609.png","size":"extralarge"}],"@attr":{"rank":"19"}},{"name":"Spirit In Black","duration":"247","playcount":"37916","listeners":"15797","mbid":"24882c73-e1bf-4814-8bd1-4fefe629695c","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Spirit+In+Black","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47615451.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47615451.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47615451.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47615451.png","size":"extralarge"}],"@attr":{"rank":"20"}},{"name":"Expendable Youth","duration":"250","playcount":"36305","listeners":"14906","mbid":"6c78619d-0712-4381-b74a-02e8fadf04f6","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Expendable+Youth","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47615451.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47615451.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47615451.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47615451.png","size":"extralarge"}],"@attr":{"rank":"21"}},{"name":"Black Magic","duration":"242","playcount":"44212","listeners":"14411","mbid":"0868bf5a-570b-43e5-b9ac-f302bc53f1bb","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Black+Magic","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/69526808.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/69526808.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/69526808.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/69526808.png","size":"extralarge"}],"@attr":{"rank":"22"}},{"name":"Skeletons Of Society","duration":"281","playcount":"35420","listeners":"14260","mbid":"13492663-2998-4c98-96b5-38c9e17e424c","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Skeletons+Of+Society","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47615451.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47615451.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47615451.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47615451.png","size":"extralarge"}],"@attr":{"rank":"23"}},{"name":"Hallowed Point","duration":"204","playcount":"33437","listeners":"14053","mbid":"3156c8f6-eb22-4fc6-97ef-77ea0c29d2a7","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Hallowed+Point","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47615451.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47615451.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47615451.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47615451.png","size":"extralarge"}],"@attr":{"rank":"24"}},{"name":"Live Undead","duration":"185","playcount":"32547","listeners":"13891","mbid":"6dd85030-5418-421b-8261-5d4941fa1bfa","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Live+Undead","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47464609.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47464609.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47464609.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47464609.png","size":"extralarge"}],"@attr":{"rank":"25"}},{"name":"Die by the Sword","duration":"216","playcount":"37076","listeners":"12954","mbid":"6d174a9b-61e6-40ae-8d40-5ef5295c585c","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Die+by+the+Sword","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/69526808.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/69526808.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/69526808.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/69526808.png","size":"extralarge"}],"@attr":{"rank":"26"}},{"name":"Hell Awaits","duration":"374","playcount":"35492","listeners":"12950","mbid":"881c7720-6411-4b6e-82d9-89c0b4924978","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Hell+Awaits","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/69450894.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/69450894.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/69450894.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/69450894.png","size":"extralarge"}],"@attr":{"rank":"27"}},{"name":"Behind The Crooked Cross","duration":"195","playcount":"31316","listeners":"12923","mbid":"8c7ba311-ad01-42dc-92dc-ee497e282f08","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Behind+The+Crooked+Cross","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47464609.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47464609.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47464609.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47464609.png","size":"extralarge"}],"@attr":{"rank":"28"}},{"name":"The Antichrist","duration":"192","playcount":"38686","listeners":"12878","mbid":"03822d93-88fc-44a0-b71f-9fb49c6be55d","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/The+Antichrist","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/54966909.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/54966909.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/54966909.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/54966909.png","size":"extralarge"}],"@attr":{"rank":"29"}},{"name":"Spill The Blood","duration":"289","playcount":"30230","listeners":"12835","mbid":"12653e64-be20-40f6-aaa1-a21b7dfcd245","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Spill+The+Blood","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47464609.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47464609.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47464609.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47464609.png","size":"extralarge"}],"@attr":{"rank":"30"}},{"name":"Aggressive Perfector","duration":"149","playcount":"32299","listeners":"12650","mbid":"08639157-a05a-4b2d-bf90-bc7b820c96c2","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Aggressive+Perfector","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/46065199.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/46065199.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/46065199.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/46065199.png","size":"extralarge"}],"@attr":{"rank":"31"}},{"name":"God Send Death","duration":"225","playcount":"31909","listeners":"12491","mbid":"299c6c53-a4af-4793-af80-b2b467c51a14","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/God+Send+Death","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/83419183.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/83419183.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83419183.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/83419183.jpg","size":"extralarge"}],"@attr":{"rank":"32"}},{"name":"World Painted Blood","duration":"353","playcount":"40287","listeners":"12326","mbid":"04e95631-9d23-44d8-aec5-c535298de79e","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/World+Painted+Blood","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/57559057.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/57559057.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/57559057.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/57559057.png","size":"extralarge"}],"@attr":{"rank":"33"}},{"name":"Born Of Fire","duration":"188","playcount":"28677","listeners":"12216","mbid":"1846c2fa-095f-40d3-a096-02875e121a59","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Born+Of+Fire","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47615451.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47615451.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47615451.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47615451.png","size":"extralarge"}],"@attr":{"rank":"34"}},{"name":"New Faith","duration":"185","playcount":"30553","listeners":"12057","mbid":"447826d7-160d-4aa7-8839-1358a3918c1a","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/New+Faith","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/83419183.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/83419183.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83419183.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/83419183.jpg","size":"extralarge"}],"@attr":{"rank":"35"}},{"name":"Ghosts Of War","duration":"234","playcount":"28047","listeners":"11857","mbid":"6aaf12b5-21b4-4f9d-a590-875d56987302","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Ghosts+Of+War","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47464609.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47464609.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47464609.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47464609.png","size":"extralarge"}],"@attr":{"rank":"36"}},{"name":"Evil Has No Boundaries","duration":"188","playcount":"31382","listeners":"11500","mbid":"151aadfc-f3f7-48f6-beab-a1a2866da2f2","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Evil+Has+No+Boundaries","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/69526808.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/69526808.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/69526808.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/69526808.png","size":"extralarge"}],"@attr":{"rank":"37"}},{"name":"Temptation","duration":"206","playcount":"25013","listeners":"11490","mbid":"2e45954a-9f26-40aa-bf0d-4b2a67a4b9ab","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Temptation","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47615451.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47615451.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47615451.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47615451.png","size":"extralarge"}],"@attr":{"rank":"38"}},{"name":"Read Between The Lies","duration":"200","playcount":"25110","listeners":"10952","mbid":"b184bf36-8bbe-4f30-85a7-b5e323e8b647","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Read+Between+The+Lies","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47464609.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47464609.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47464609.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47464609.png","size":"extralarge"}],"@attr":{"rank":"39"}},{"name":"Cleanse The Soul","duration":"182","playcount":"24273","listeners":"10824","mbid":"3153e851-c416-4f8c-abb2-81c6b8ac1159","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Cleanse+The+Soul","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/47464609.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/47464609.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/47464609.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/47464609.png","size":"extralarge"}],"@attr":{"rank":"40"}},{"name":"Threshold","duration":"148","playcount":"26669","listeners":"9946","mbid":"977dbb48-63e3-4757-90f9-94c855b60f79","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Threshold","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19778897.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19778897.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19778897.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19778897.jpg","size":"extralarge"}],"@attr":{"rank":"41"}},{"name":"Show No Mercy","duration":"181","playcount":"26965","listeners":"9883","mbid":"0219b49f-5c2a-43b8-a2ce-98754ef22b11","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Show+No+Mercy","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/69526808.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/69526808.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/69526808.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/69526808.png","size":"extralarge"}],"@attr":{"rank":"42"}},{"name":"Cast Down","duration":"205","playcount":"23375","listeners":"9830","mbid":"3c56f8c6-6bd9-457b-9e1c-51c0c4f1d199","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Cast+Down","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19778897.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19778897.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19778897.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19778897.jpg","size":"extralarge"}],"@attr":{"rank":"43"}},{"name":"Jihad","duration":"271","playcount":"27735","listeners":"9807","mbid":"26a7900f-3fcb-42ca-a721-0d66b4337f3f","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Jihad","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/cdn.last.fm\/flatness\/catalogue\/noimage\/2\/default_album_medium.png","size":"small"},{"#text":"http:\/\/cdn.last.fm\/flatness\/catalogue\/noimage\/2\/default_album_medium.png","size":"medium"},{"#text":"http:\/\/cdn.last.fm\/flatness\/catalogue\/noimage\/2\/default_album_medium.png","size":"large"},{"#text":"http:\/\/cdn.last.fm\/flatness\/catalogue\/noimage\/2\/default_album_medium.png","size":"extralarge"}],"@attr":{"rank":"44"}},{"name":"Exile","duration":"234","playcount":"23394","listeners":"9731","mbid":"0c5661a8-a28f-4502-9b6c-6677217581da","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Exile","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19778897.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19778897.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19778897.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19778897.jpg","size":"extralarge"}],"@attr":{"rank":"45"}},{"name":"Eyes Of The Insane","duration":"201","playcount":"23172","listeners":"9545","mbid":"09e443ec-94d7-444b-acbe-2894a730a506","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Eyes+Of+The+Insane","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/cdn.last.fm\/flatness\/catalogue\/noimage\/2\/default_album_medium.png","size":"small"},{"#text":"http:\/\/cdn.last.fm\/flatness\/catalogue\/noimage\/2\/default_album_medium.png","size":"medium"},{"#text":"http:\/\/cdn.last.fm\/flatness\/catalogue\/noimage\/2\/default_album_medium.png","size":"large"},{"#text":"http:\/\/cdn.last.fm\/flatness\/catalogue\/noimage\/2\/default_album_medium.png","size":"extralarge"}],"@attr":{"rank":"46"}},{"name":"Cult","duration":"275","playcount":"24600","listeners":"9216","mbid":"0e275a10-a871-4160-b272-49db01a51e6e","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Cult","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/79002991.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/79002991.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/79002991.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/79002991.jpg","size":"extralarge"}],"@attr":{"rank":"47"}},{"name":"Here Comes The Pain","duration":"275","playcount":"22107","listeners":"9206","mbid":"124ea05c-2202-444c-a343-80a1acd4482a","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Here+Comes+The+Pain","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/19778897.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/19778897.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/19778897.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/19778897.jpg","size":"extralarge"}],"@attr":{"rank":"48"}},{"name":"Tormentor","duration":"224","playcount":"22828","listeners":"9117","mbid":"0c9a8c93-c395-47cd-ae9c-443a751f75cc","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Tormentor","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/69526808.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/69526808.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/69526808.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/69526808.png","size":"extralarge"}],"@attr":{"rank":"49"}},{"name":"Fight Till Death","duration":"216","playcount":"22112","listeners":"9099","mbid":"34d4b777-7cd0-4749-af2f-3f9fa7afa7ec","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Fight+Till+Death","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/69526808.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/69526808.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/69526808.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/69526808.png","size":"extralarge"}],"@attr":{"rank":"50"}}],"@attr":{"artist":"Slayer","page":"1","perPage":"50","totalPages":"20","total":"1000"}}},{})
    var scrobbler = new scribble('a','a','a','a')
    scrobbler.GetArtistTopTracks(song.artist, function(ret) {
      ret.should.be.a('object').and.have.property('toptracks')
      endTest()
    })
  })
}) // [Artist Top Tracks]


/*<><><><><><><><><><><><><><><><><><><>
                TRACK
<><><><><><><><><><><><><><><><><><><>*/
describe('[Similar Songs]', function() {
  // 11
  it('should return similar songs', function(endTest) {
    // mock request
    nock('http://ws.audioscrobbler.com:80')
        .get('/2.0/?method=track.getSimilar&artist=' + song.artist + '&track=' + song.track + '&api_key=a&format=json&limit=50')
        .reply(200, {"similartracks":{"track":[{"name":"God Send Death","playcount":"538142","mbid":"299c6c53-a4af-4793-af80-b2b467c51a14","match":"1","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/God+Send+Death","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/83419183.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/83419183.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83419183.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/83419183.jpg","size":"extralarge"}]},{"name":"New Faith","playcount":"517467","mbid":"447826d7-160d-4aa7-8839-1358a3918c1a","match":"0.977871","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/New+Faith","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/83419183.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/83419183.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83419183.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/83419183.jpg","size":"extralarge"}]},{"name":"Roots Bloody Roots","playcount":"1136256","mbid":"4f4b4f96-ac51-4ed5-8097-633b6a8ef9f6","match":"0.235589","url":"http:\/\/www.last.fm\/music\/Sepultura\/_\/Roots+Bloody+Roots","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Sepultura","mbid":"1d93c839-22e7-4f76-ad84-d27039efc048","url":"http:\/\/www.last.fm\/music\/Sepultura"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/63137357.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/63137357.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/63137357.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/63137357.png","size":"extralarge"}]},{"name":"Bonded By Blood","playcount":"388643","mbid":"2c6431b6-446f-493e-9211-bf570ceadf7c","match":"0.214123","url":"http:\/\/www.last.fm\/music\/Exodus\/_\/Bonded+By+Blood","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Exodus","mbid":"1067b675-7637-48e4-9085-0378526b357a","url":"http:\/\/www.last.fm\/music\/Exodus"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/51045499.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/51045499.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/51045499.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/51045499.png","size":"extralarge"}]},{"name":"Peace Sells","playcount":"1777577","mbid":"6e988221-6b1c-49d7-8fae-be73503177df","match":"0.211641","url":"http:\/\/www.last.fm\/music\/Megadeth\/_\/Peace+Sells","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Megadeth","mbid":"a9044915-8be3-4c7e-b11f-9e2d2ea0a91e","url":"http:\/\/www.last.fm\/music\/Megadeth"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/61105685.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/61105685.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/61105685.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/61105685.jpg","size":"extralarge"}]},{"name":"Tornado Of Souls","playcount":"1520075","mbid":"0cc0dd17-94a0-44b1-b6a9-755229c164f5","match":"0.211186","url":"http:\/\/www.last.fm\/music\/Megadeth\/_\/Tornado+Of+Souls","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Megadeth","mbid":"a9044915-8be3-4c7e-b11f-9e2d2ea0a91e","url":"http:\/\/www.last.fm\/music\/Megadeth"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/9660177.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/9660177.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/9660177.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/9660177.jpg","size":"extralarge"}]},{"name":"Walk","playcount":"2526905","mbid":"4d7a12cd-72d2-45f6-8734-fd45f30eab9c","match":"0.210912","url":"http:\/\/www.last.fm\/music\/Pantera\/_\/Walk","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Pantera","mbid":"541f16f5-ad7a-428e-af89-9fa1b16d3c9c","url":"http:\/\/www.last.fm\/music\/Pantera"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/69350472.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/69350472.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/69350472.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/69350472.png","size":"extralarge"}]},{"name":"Territory","playcount":"723089","mbid":"3a9e61b5-1c00-4f5a-847a-44640d7232fa","match":"0.210567","url":"http:\/\/www.last.fm\/music\/Sepultura\/_\/Territory","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Sepultura","mbid":"1d93c839-22e7-4f76-ad84-d27039efc048","url":"http:\/\/www.last.fm\/music\/Sepultura"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/75430090.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/75430090.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/75430090.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/75430090.png","size":"extralarge"}]},{"name":"Creeping Death","playcount":"2543658","mbid":"4473620d-2b63-4c29-81f6-f8a6c87b2f92","match":"0.203759","url":"http:\/\/www.last.fm\/music\/Metallica\/_\/Creeping+Death","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Metallica","mbid":"65f4f0c5-ef9e-490c-aee3-909e7ae6b2ab","url":"http:\/\/www.last.fm\/music\/Metallica"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/77093442.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/77093442.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/77093442.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/77093442.png","size":"extralarge"}]},{"name":"Battery","playcount":"3631304","mbid":"85a12613-64b0-4067-aba6-2b13ddb4aca4","match":"0.200406","url":"http:\/\/www.last.fm\/music\/Metallica\/_\/Battery","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Metallica","mbid":"65f4f0c5-ef9e-490c-aee3-909e7ae6b2ab","url":"http:\/\/www.last.fm\/music\/Metallica"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/71388652.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/71388652.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/71388652.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/71388652.png","size":"extralarge"}]},{"name":"I Am The Law","playcount":"641949","mbid":"5c05b064-77fa-40ae-98ee-e43bf4f31853","match":"0.19666","url":"http:\/\/www.last.fm\/music\/Anthrax\/_\/I+Am+The+Law","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Anthrax","mbid":"315126eb-86bc-475c-a308-b8edfdd6c08c","url":"http:\/\/www.last.fm\/music\/Anthrax"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/83633229.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/83633229.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83633229.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/83633229.png","size":"extralarge"}]},{"name":"Madhouse","playcount":"818221","mbid":"004b4174-c77e-4be3-8e36-979ada6a77b3","match":"0.196196","url":"http:\/\/www.last.fm\/music\/Anthrax\/_\/Madhouse","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Anthrax","mbid":"315126eb-86bc-475c-a308-b8edfdd6c08c","url":"http:\/\/www.last.fm\/music\/Anthrax"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/82383371.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/82383371.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/82383371.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/82383371.png","size":"extralarge"}]},{"name":"Enemy Of God","playcount":"476700","mbid":"91bcbf95-4da3-4598-a574-cd6af9c44230","match":"0.195262","url":"http:\/\/www.last.fm\/music\/Kreator\/_\/Enemy+Of+God","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Kreator","mbid":"39822e8d-f24e-4f07-b51b-28b22e59fbdb","url":"http:\/\/www.last.fm\/music\/Kreator"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/77912112.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/77912112.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/77912112.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/77912112.jpg","size":"extralarge"}]},{"name":"Fucking Hostile","playcount":"1454718","mbid":"02852346-e139-4d41-afc1-b2983f4be37a","match":"0.19322","url":"http:\/\/www.last.fm\/music\/Pantera\/_\/Fucking+Hostile","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Pantera","mbid":"541f16f5-ad7a-428e-af89-9fa1b16d3c9c","url":"http:\/\/www.last.fm\/music\/Pantera"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/69350498.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/69350498.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/69350498.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/69350498.png","size":"extralarge"}]},{"name":"Phantom Antichrist","playcount":"102660","mbid":"2f7858bd-54c4-4bb7-98b3-20940f6be988","match":"0.190638","url":"http:\/\/www.last.fm\/music\/Kreator\/_\/Phantom+Antichrist","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Kreator","mbid":"39822e8d-f24e-4f07-b51b-28b22e59fbdb","url":"http:\/\/www.last.fm\/music\/Kreator"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/76881312.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/76881312.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/76881312.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/76881312.png","size":"extralarge"}]},{"name":"A Lesson in Violence","playcount":"260631","mbid":"b35d898e-bc07-4086-8b19-7df48d9b3b15","match":"0.183478","url":"http:\/\/www.last.fm\/music\/Exodus\/_\/A+Lesson+in+Violence","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Exodus","mbid":"1067b675-7637-48e4-9085-0378526b357a","url":"http:\/\/www.last.fm\/music\/Exodus"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/51045499.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/51045499.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/51045499.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/51045499.png","size":"extralarge"}]},{"name":"Rise Up","playcount":"62618","mbid":"faaae27c-c522-49be-8916-9b20c31687ef","match":"0.166417","url":"http:\/\/www.last.fm\/music\/Testament\/_\/Rise+Up","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Testament","mbid":"05106775-5d45-4131-aecc-1177f813ba11","url":"http:\/\/www.last.fm\/music\/Testament"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/83786719.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/83786719.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83786719.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/83786719.jpg","size":"extralarge"}]},{"name":"Agent Orange","playcount":"273083","mbid":"1b233d04-8e11-4bfb-8951-e1aa4962c35e","match":"0.160688","url":"http:\/\/www.last.fm\/music\/Sodom\/_\/Agent+Orange","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Sodom","mbid":"2251718f-7b5c-4a92-9267-895125c594d3","url":"http:\/\/www.last.fm\/music\/Sodom"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/73851910.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/73851910.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/73851910.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/73851910.png","size":"extralarge"}]},{"name":"Laid to Rest","playcount":"1983216","mbid":"09799fc1-083c-4b57-b9dd-bc340583d232","match":"0.157873","url":"http:\/\/www.last.fm\/music\/Lamb+of+God\/_\/Laid+to+Rest","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Lamb of God","mbid":"298909e4-ebcb-47b8-95e9-cc53b087fc65","url":"http:\/\/www.last.fm\/music\/Lamb+of+God"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/58117585.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/58117585.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/58117585.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/58117585.png","size":"extralarge"}]},{"name":"Over the Wall","playcount":"413963","mbid":"17109352-7c7c-4c6f-b146-b03bd9fffca7","match":"0.155234","url":"http:\/\/www.last.fm\/music\/Testament\/_\/Over+the+Wall","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Testament","mbid":"05106775-5d45-4131-aecc-1177f813ba11","url":"http:\/\/www.last.fm\/music\/Testament"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/44917003.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/44917003.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/44917003.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/44917003.png","size":"extralarge"}]},{"name":"Locust","playcount":"347986","mbid":"f8c46527-b3ef-432e-9867-a1cd23a07a19","match":"0.151611","url":"http:\/\/www.last.fm\/music\/Machine+Head\/_\/Locust","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Machine Head","mbid":"22a1f4d7-81ff-4216-ac06-3387da6bc118","url":"http:\/\/www.last.fm\/music\/Machine+Head"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/80046367.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/80046367.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/80046367.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/80046367.png","size":"extralarge"}]},{"name":"Aesthetics of Hate","playcount":"913603","mbid":"0dd71068-e52b-4df5-8e87-40c2f5ec4945","match":"0.150743","url":"http:\/\/www.last.fm\/music\/Machine+Head\/_\/Aesthetics+of+Hate","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Machine Head","mbid":"22a1f4d7-81ff-4216-ac06-3387da6bc118","url":"http:\/\/www.last.fm\/music\/Machine+Head"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/83395859.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/83395859.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83395859.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/83395859.jpg","size":"extralarge"}]},{"name":"Alison Hell","playcount":"367339","mbid":"2f2335c4-a8da-40ca-b461-c06227cba2b2","match":"0.142129","url":"http:\/\/www.last.fm\/music\/Annihilator\/_\/Alison+Hell","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Annihilator","mbid":"5dadb7e6-864f-4107-b6d6-dfd1b6d50830","url":"http:\/\/www.last.fm\/music\/Annihilator"},"image":[{"#text":"http:\/\/images.amazon.com\/images\/P\/B00008KH1H.01._SCMZZZZZZZ_.jpg","size":"small"},{"#text":"http:\/\/images.amazon.com\/images\/P\/B00008KH1H.01._SCMZZZZZZZ_.jpg","size":"medium"},{"#text":"http:\/\/images.amazon.com\/images\/P\/B00008KH1H.01._SCMZZZZZZZ_.jpg","size":"large"},{"#text":"http:\/\/images.amazon.com\/images\/P\/B00008KH1H.01._SCMZZZZZZZ_.jpg","size":"extralarge"}]},{"name":"Redneck","playcount":"1619153","mbid":"47516dee-894c-423c-b046-2ab181470871","match":"0.142041","url":"http:\/\/www.last.fm\/music\/Lamb+of+God\/_\/Redneck","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Lamb of God","mbid":"298909e4-ebcb-47b8-95e9-cc53b087fc65","url":"http:\/\/www.last.fm\/music\/Lamb+of+God"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/82087209.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/82087209.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/82087209.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/82087209.png","size":"extralarge"}]},{"name":"M-16","playcount":"147659","mbid":"41037d95-46ad-424e-a32a-70db73831611","match":"0.138126","url":"http:\/\/www.last.fm\/music\/Sodom\/_\/M-16","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Sodom","mbid":"2251718f-7b5c-4a92-9267-895125c594d3","url":"http:\/\/www.last.fm\/music\/Sodom"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/73853188.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/73853188.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/73853188.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/73853188.png","size":"extralarge"}]},{"name":"Painkiller","playcount":"1578814","mbid":"1a0d3aa9-9c41-4607-adbc-77e08a6290f4","match":"0.129574","url":"http:\/\/www.last.fm\/music\/Judas+Priest\/_\/Painkiller","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Judas Priest","mbid":"6b335658-22c8-485d-93de-0bc29a1d0349","url":"http:\/\/www.last.fm\/music\/Judas+Priest"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/72343940.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/72343940.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/72343940.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/72343940.png","size":"extralarge"}]},{"name":"Black Metal","playcount":"357497","mbid":"f037eb15-d2c0-4962-b5d5-7ba3b9d9fba9","match":"0.112604","url":"http:\/\/www.last.fm\/music\/Venom\/_\/Black+Metal","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Venom","mbid":"5ddddef1-fd5a-4ca8-8e89-df4adff4239b","url":"http:\/\/www.last.fm\/music\/Venom"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/15648251.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/15648251.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/15648251.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/15648251.jpg","size":"extralarge"}]},{"name":"Thrash Till Death","playcount":"66639","mbid":"b8d5ccb9-0262-4a85-ae7b-68d7d792bfed","match":"0.109661","url":"http:\/\/www.last.fm\/music\/Destruction\/_\/Thrash+Till+Death","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Destruction","mbid":"371f152d-1643-4b54-b32b-dd13d4c23442","url":"http:\/\/www.last.fm\/music\/Destruction"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/34604525.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/34604525.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/34604525.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/34604525.jpg","size":"extralarge"}]},{"name":"Hate Is My Fuel","playcount":"28326","mbid":"264c1c8d-8a5c-492b-ab8b-7cff8034b94c","match":"0.100955","url":"http:\/\/www.last.fm\/music\/Destruction\/_\/Hate+Is+My+Fuel","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Destruction","mbid":"371f152d-1643-4b54-b32b-dd13d4c23442","url":"http:\/\/www.last.fm\/music\/Destruction"}},{"name":"Elimination","playcount":"135689","mbid":"1e075f3e-a802-4cfd-82f8-9443de70d503","match":"0.100664","url":"http:\/\/www.last.fm\/music\/Overkill\/_\/Elimination","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Overkill","mbid":"97fa7dd1-4706-4fad-b6ff-e099a37837fc","url":"http:\/\/www.last.fm\/music\/Overkill"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/20287225.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/20287225.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/20287225.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/20287225.jpg","size":"extralarge"}]},{"name":"King of the Kill","playcount":"197786","mbid":"880cba0c-410c-4fa0-9aa1-d9ac9798bc36","match":"0.100567","url":"http:\/\/www.last.fm\/music\/Annihilator\/_\/King+of+the+Kill","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Annihilator","mbid":"5dadb7e6-864f-4107-b6d6-dfd1b6d50830","url":"http:\/\/www.last.fm\/music\/Annihilator"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/44103421.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/44103421.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/44103421.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/44103421.jpg","size":"extralarge"}]},{"name":"Come And Get It","playcount":"32745","mbid":"9dba7c62-a744-432d-b18d-c247bca4b755","match":"0.0994264","url":"http:\/\/www.last.fm\/music\/Overkill\/_\/Come+And+Get+It","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Overkill","mbid":"97fa7dd1-4706-4fad-b6ff-e099a37837fc","url":"http:\/\/www.last.fm\/music\/Overkill"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/80653991.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/80653991.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/80653991.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/80653991.jpg","size":"extralarge"}]},{"name":"In Waves","playcount":"524891","mbid":"52ca5402-e05b-47be-b361-a0d1b5ea9f43","match":"0.0987073","url":"http:\/\/www.last.fm\/music\/Trivium\/_\/In+Waves","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Trivium","mbid":"1f5ff245-2837-4d4a-a609-e93e544478c3","url":"http:\/\/www.last.fm\/music\/Trivium"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/79808771.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/79808771.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/79808771.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/79808771.png","size":"extralarge"}]},{"name":"Cult","playcount":"31964","mbid":"1a5a8f06-9587-4728-8a7f-53f388d61c86","match":"0.0935189","url":"http:\/\/www.last.fm\/music\/Evile\/_\/Cult","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Evile","mbid":"5540499f-8ab5-404f-a781-e959b1dc2778","url":"http:\/\/www.last.fm\/music\/Evile"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/69626662.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/69626662.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/69626662.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/69626662.png","size":"extralarge"}]},{"name":"The Art Of Partying","playcount":"296502","mbid":"19c24844-80a8-40ec-a99b-1b0feb184af2","match":"0.091887","url":"http:\/\/www.last.fm\/music\/Municipal+Waste\/_\/The+Art+Of+Partying","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Municipal Waste","mbid":"b5a31e9b-1fa2-45f9-91f5-b3a25fb38038","url":"http:\/\/www.last.fm\/music\/Municipal+Waste"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/75115378.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/75115378.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/75115378.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/75115378.png","size":"extralarge"}]},{"name":"Overkill","playcount":"857664","mbid":"97b595eb-ab8b-4830-93c9-02489ed81fd7","match":"0.0906829","url":"http:\/\/www.last.fm\/music\/Mot%C3%B6rhead\/_\/Overkill","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Mot\u00f6rhead","mbid":"57961a97-3796-4bf7-9f02-a985a8979ae9","url":"http:\/\/www.last.fm\/music\/Mot%C3%B6rhead"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/81615437.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/81615437.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/81615437.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/81615437.jpg","size":"extralarge"}]},{"name":"Babylon","playcount":"330045","mbid":"0f9bae99-93b2-472e-9f79-a92a327a46d9","match":"0.0899575","url":"http:\/\/www.last.fm\/music\/Soulfly\/_\/Babylon","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Soulfly","mbid":"832a43c7-aa7d-439b-a6b4-4f1afa671c24","url":"http:\/\/www.last.fm\/music\/Soulfly"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/66237288.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/66237288.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/66237288.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/66237288.png","size":"extralarge"}]},{"name":"Welcome to Hell","playcount":"177181","mbid":"02b233a6-6e85-469a-a5d5-d01aba4b4344","match":"0.084133","url":"http:\/\/www.last.fm\/music\/Venom\/_\/Welcome+to+Hell","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Venom","mbid":"5ddddef1-fd5a-4ca8-8e89-df4adff4239b","url":"http:\/\/www.last.fm\/music\/Venom"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/15648043.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/15648043.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/15648043.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/15648043.jpg","size":"extralarge"}]},{"name":"Thrasher","playcount":"190341","mbid":"31097294-20a3-4e63-931d-e82446bf20af","match":"0.0827658","url":"http:\/\/www.last.fm\/music\/Evile\/_\/Thrasher","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Evile","mbid":"5540499f-8ab5-404f-a781-e959b1dc2778","url":"http:\/\/www.last.fm\/music\/Evile"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/55457927.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/55457927.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/55457927.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/55457927.jpg","size":"extralarge"}]},{"name":"Kill The Gringo","playcount":"12504","mbid":"444d97c1-9316-4fb7-91c8-e83cb2fc9d15","match":"0.0806246","url":"http:\/\/www.last.fm\/music\/Acid+Drinkers\/_\/Kill+The+Gringo","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Acid Drinkers","mbid":"3e103047-217b-496b-b54e-1e2db4dd58b3","url":"http:\/\/www.last.fm\/music\/Acid+Drinkers"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/83800969.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/83800969.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83800969.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/83800969.png","size":"extralarge"}]},{"name":"Inflikted","playcount":"368563","mbid":"3541a040-8370-4c7b-bbfd-b2c093a5fb47","match":"0.0797412","url":"http:\/\/www.last.fm\/music\/Cavalera+Conspiracy\/_\/Inflikted","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Cavalera Conspiracy","mbid":"5bd1060b-8608-4817-93ca-207f7c828e2f","url":"http:\/\/www.last.fm\/music\/Cavalera+Conspiracy"},"image":[{"#text":"http:\/\/cdn.last.fm\/flatness\/catalogue\/noimage\/2\/default_album_medium.png","size":"small"},{"#text":"http:\/\/cdn.last.fm\/flatness\/catalogue\/noimage\/2\/default_album_medium.png","size":"medium"},{"#text":"http:\/\/cdn.last.fm\/flatness\/catalogue\/noimage\/2\/default_album_medium.png","size":"large"},{"#text":"http:\/\/cdn.last.fm\/flatness\/catalogue\/noimage\/2\/default_album_medium.png","size":"extralarge"}]},{"name":"Entrance Of The Conflagration","playcount":"578868","mbid":"f8138d3b-9feb-4ee8-a7ad-616d14f56fc6","match":"0.0780002","url":"http:\/\/www.last.fm\/music\/Trivium\/_\/Entrance+Of+The+Conflagration","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Trivium","mbid":"1f5ff245-2837-4d4a-a609-e93e544478c3","url":"http:\/\/www.last.fm\/music\/Trivium"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/80338867.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/80338867.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/80338867.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/80338867.jpg","size":"extralarge"}]},{"name":"The Trick","playcount":"12294","mbid":"a9ef8134-e25a-4c95-adb8-8c75373c8e27","match":"0.0776106","url":"http:\/\/www.last.fm\/music\/Acid+Drinkers\/_\/The+Trick","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Acid Drinkers","mbid":"3e103047-217b-496b-b54e-1e2db4dd58b3","url":"http:\/\/www.last.fm\/music\/Acid+Drinkers"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/83800969.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/83800969.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83800969.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/83800969.png","size":"extralarge"}]},{"name":"Terrorize","playcount":"251348","mbid":"3e17dd3e-749f-4d4f-9bd0-2bad86b97ed6","match":"0.0755338","url":"http:\/\/www.last.fm\/music\/Cavalera+Conspiracy\/_\/Terrorize","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Cavalera Conspiracy","mbid":"5bd1060b-8608-4817-93ca-207f7c828e2f","url":"http:\/\/www.last.fm\/music\/Cavalera+Conspiracy"}},{"name":"Blood Fire War Hate","playcount":"187567","mbid":"6f84f656-2f60-4bef-a43e-1b801cac9720","match":"0.0722627","url":"http:\/\/www.last.fm\/music\/Soulfly\/_\/Blood+Fire+War+Hate","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Soulfly","mbid":"832a43c7-aa7d-439b-a6b4-4f1afa671c24","url":"http:\/\/www.last.fm\/music\/Soulfly"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/66237388.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/66237388.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/66237388.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/66237388.png","size":"extralarge"}]},{"name":"Die With A Beer In Your Hand","playcount":"59083","mbid":"84c7bb50-7e1a-44b9-8c82-5b405e86f43c","match":"0.0691464","url":"http:\/\/www.last.fm\/music\/Tankard\/_\/Die+With+A+Beer+In+Your+Hand","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Tankard","mbid":"619ce797-ddb3-4411-9e42-30eafe2ca5ec","url":"http:\/\/www.last.fm\/music\/Tankard"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/82153371.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/82153371.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/82153371.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/82153371.jpg","size":"extralarge"}]},{"name":"Headbanger Face Rip","playcount":"252449","mbid":"116b5fef-b9f2-43e6-af33-fde263e97c7b","match":"0.0672847","url":"http:\/\/www.last.fm\/music\/Municipal+Waste\/_\/Headbanger+Face+Rip","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Municipal Waste","mbid":"b5a31e9b-1fa2-45f9-91f5-b3a25fb38038","url":"http:\/\/www.last.fm\/music\/Municipal+Waste"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/75115378.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/75115378.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/75115378.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/75115378.png","size":"extralarge"}]},{"name":"Clouds Over California","playcount":"749143","mbid":"f46fc469-7828-4ffc-9942-43054dc06d30","match":"0.0630182","url":"http:\/\/www.last.fm\/music\/DevilDriver\/_\/Clouds+Over+California","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"DevilDriver","mbid":"1aded574-9e16-4648-861a-24fd6f1c309e","url":"http:\/\/www.last.fm\/music\/DevilDriver"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/85402451.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/85402451.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/85402451.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/85402451.jpg","size":"extralarge"}]},{"name":"\u0141za dla cieni\u00f3w minionych","playcount":"331477","mbid":"1cf48498-e403-4849-9fd8-cd2419ffe955","match":"0.0623792","url":"http:\/\/www.last.fm\/music\/Kat\/_\/%C5%81za+dla+cieni%C3%B3w+minionych","streamable":{"#text":"0","fulltrack":"0"},"duration":"214000","artist":{"name":"Kat","mbid":"5bba9d6a-5f91-4a00-9bfb-9c8257ddedbb","url":"http:\/\/www.last.fm\/music\/Kat"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/60545637.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/60545637.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/60545637.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/60545637.jpg","size":"extralarge"}]},{"name":"Prepare For Attack","playcount":"34007","mbid":"a821e9a4-29b0-4be5-9311-6120aa17047b","match":"0.0574948","url":"http:\/\/www.last.fm\/music\/Havok\/_\/Prepare+For+Attack","streamable":{"#text":"1","fulltrack":"0"},"duration":"214000","artist":{"name":"Havok","mbid":"4c761f9b-f4b9-462c-9f0e-f247e6152767","url":"http:\/\/www.last.fm\/music\/Havok"},"image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/70227756.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/70227756.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/70227756.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/70227756.jpg","size":"extralarge"}]}],"@attr":{"track":"Disciple","artist":"Slayer"}}},{})
    var scrobbler = new scribble('a','a','a','a')
    scrobbler.GetSimilarSongs(song, function(ret) {
      ret.should.be.a('object').and.have.property('similartracks')
      endTest()
    })
  })
}) // [Similar Songs]

describe('[Track Get Info]', function() {
  it('should return an object of track information', function(endTest) {
    // mock request
    nock('http://ws.audioscrobbler.com:80')
        .get('/2.0/?method=track.getInfo&api_key=a&artist=' + song.artist + '&track=' + song.track + '&format=json')
        .reply(200, {"track":{"id":"33934","name":"Disciple","mbid":"4899b9b5-d45c-405e-b9b4-970416ad9aa7","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Disciple","duration":"214000","streamable":{"#text":"0","fulltrack":"0"},"listeners":"154493","playcount":"1005970","artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"album":{"artist":"Slayer","title":"Soundtrack to the Apocalypse (disc 2)","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/Soundtrack+to+the+Apocalypse+(disc+2)","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/83419183.jpg","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/126\/83419183.jpg","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/174s\/83419183.jpg","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/83419183.jpg","size":"extralarge"}],"@attr":{"position":"12"}},"toptags":{"tag":[{"name":"thrash metal","url":"http:\/\/www.last.fm\/tag\/thrash%20metal"},{"name":"metal","url":"http:\/\/www.last.fm\/tag\/metal"},{"name":"slayer","url":"http:\/\/www.last.fm\/tag\/slayer"},{"name":"heavy metal","url":"http:\/\/www.last.fm\/tag\/heavy%20metal"},{"name":"speed metal","url":"http:\/\/www.last.fm\/tag\/speed%20metal"}]}}},{})
    var scrobbler = new scribble('a','a','a','a')
    scrobbler.GetTrackInfo(song, function(ret) {
      ret.should.be.a('object').and.have.property('track')
      endTest()
    })
  })
}) // [Track Get Info]

/*<><><><><><><><><><><><><><><><><><><>
                ALBUM
<><><><><><><><><><><><><><><><><><><>*/
describe('[Album Get Info]', function() {
  it('should return an object of album info', function(endTest) {
    // mock request
    nock('http://ws.audioscrobbler.com:80')
        .get('2.0/?method=album.getinfo&api_key=a&artist=' + song.artist + '&album=' + song.album.replace(/\s/g, '%20') + '&format=json')
        .reply(200, {"album":{"name":"God Hates Us All","artist":"Slayer","id":"6416","mbid":"1c233932-faeb-48a7-96a5-a79871522e40","url":"http:\/\/www.last.fm\/music\/Slayer\/God+Hates+Us+All","releasedate":"    25 Oct 2001, 00:00","image":[{"#text":"http:\/\/userserve-ak.last.fm\/serve\/34s\/57560087.png","size":"small"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/64s\/57560087.png","size":"medium"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/174s\/57560087.png","size":"large"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/300x300\/57560087.png","size":"extralarge"},{"#text":"http:\/\/userserve-ak.last.fm\/serve\/_\/57560087\/God+Hates+Us+All+High+quality+PNG.png","size":"mega"}],"listeners":"236966","playcount":"5675111","tracks":{"track":[{"name":"Darkness Of Chirst (Album Version (Explicit))","duration":"90","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Darkness+Of+Chirst+(Album+Version+(Explicit))","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"@attr":{"rank":"1"}},{"name":"Disciple (Album Version (Explicit))","duration":"216","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Disciple+(Album+Version+(Explicit))","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"@attr":{"rank":"2"}},{"name":"God Send Death (Album Version (Explicit))","duration":"226","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/God+Send+Death+(Album+Version+(Explicit))","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"@attr":{"rank":"3"}},{"name":"New Faith (Album Version (Explicit))","duration":"186","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/New+Faith+(Album+Version+(Explicit))","streamable":{"#text":"1","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"@attr":{"rank":"4"}},{"name":"Cast Down (Album Version (Explicit))","duration":"207","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Cast+Down+(Album+Version+(Explicit))","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"@attr":{"rank":"5"}},{"name":"Threshold (Album Version (Explicit))","duration":"149","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Threshold+(Album+Version+(Explicit))","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"@attr":{"rank":"6"}},{"name":"Exile (Album Version (Explicit))","duration":"236","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Exile+(Album+Version+(Explicit))","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"@attr":{"rank":"7"}},{"name":"Seven Faces (Album Version (Explicit))","duration":"222","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Seven+Faces+(Album+Version+(Explicit))","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"@attr":{"rank":"8"}},{"name":"Bloodline (Album Version (Explicit))","duration":"217","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Bloodline+(Album+Version+(Explicit))","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"@attr":{"rank":"9"}},{"name":"Deviance (Album Version (Explicit))","duration":"189","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Deviance+(Album+Version+(Explicit))","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"@attr":{"rank":"10"}},{"name":"War Zone (Album Version (Explicit))","duration":"165","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/War+Zone+(Album+Version+(Explicit))","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"@attr":{"rank":"11"}},{"name":"Here Comes The Pain (Album Version (Explicit))","duration":"272","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Here+Comes+The+Pain+(Album+Version+(Explicit))","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"@attr":{"rank":"12"}},{"name":"Payback (Album Version (Explicit))","duration":"185","mbid":"","url":"http:\/\/www.last.fm\/music\/Slayer\/_\/Payback+(Album+Version+(Explicit))","streamable":{"#text":"0","fulltrack":"0"},"artist":{"name":"Slayer","mbid":"72de5171-38cf-4734-bc8a-6ac374dea523","url":"http:\/\/www.last.fm\/music\/Slayer"},"@attr":{"rank":"13"}}]},"toptags":{"tag":[{"name":"thrash metal","url":"http:\/\/www.last.fm\/tag\/thrash%20metal"},{"name":"albums i own","url":"http:\/\/www.last.fm\/tag\/albums%20i%20own"},{"name":"metal","url":"http:\/\/www.last.fm\/tag\/metal"},{"name":"speed metal","url":"http:\/\/www.last.fm\/tag\/speed%20metal"},{"name":"heavy metal","url":"http:\/\/www.last.fm\/tag\/heavy%20metal"}]}}},{})
    var scrobbler = new scribble('a','a','a','a')
    scrobbler.GetAlbumInfo(song, function(ret) {
      ret.should.be.a('object').and.have.property('album')
      endTest()
    })
  })
}) // [Album Get Info]